package com.zfire.mall.manager.logic.k3;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.zfire.mall.manager.bean.k3.RequestApiBean;
import com.zfire.mall.manager.commonmapper.K3Mapper;
import com.zfire.mall.manager.constant.KingDeeCons;
import com.zfire.mall.manager.plus.entity.KingDeeCategory;
import com.zfire.mall.manager.plus.entity.KingDeeMaterial;
import com.zfire.mall.manager.plus.entity.ProductUpdPriceBillItem;
import com.zfire.mall.manager.plus.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Slf4j
public class K3MaterialLogic {


    @Autowired
    K3Logic k3Logic;
    @Resource
    KingDeeMaterialService kingDeeMaterialService;
    @Autowired
    KingDeeWalletService kingDeeWalletService;
    @Autowired
    SaleTypeService saleTypeService;
    @Autowired
    K3Mapper k3Mapper;

    @Transactional
    public void allSync(String orgId, String orgNumber, String startTime, String endTime) throws Exception {
        // 调用物料接口
        RequestApiBean request = k3Logic.getInstance(orgId, orgNumber, KingDeeCons.MATERIAL_INFO, startTime, endTime);
        String filterStr = "FDocumentStatus = 'C' AND FUseOrgId.FNUMBER = %s "+ k3Logic.supplySql(request.getStartTime(), request.getEndTime());
        request.setFieldKeys("FMATERIALID,FNumber,FName,FUseOrgId,FUseOrgId.FNUMBER,FUseOrgId.FName, FCategoryID, FCategoryID.FNUMBER,FCategoryID.FName, FSpecification,"
                + "FBaseUnitId.FNumber,FBaseUnitId.FName,FStoreUnitID.FNumber,FStoreUnitID.FName,FSalePriceUnitId.FNumber,FSalePriceUnitId.FName,FPurchasePriceUnitId.FNumber,FPurchasePriceUnitId.FName,FTaxRateId.FTaxRate,"
                + "FMaterialGroup.FNumber,FIsInventory,FIsSale,FIsPurchase,FLENGTH,FVOLUME,FWIDTH,FHEIGHT,FOldNumber,F_tmt_Text,F_njdm_Text,F_njxh_Text,F_wjdm_Text,F_wjxh_Text,"
                + "F_pjdm_Text,F_pjxh1_Text,F_pjcspdm_Text,F_pjxh2_Text,FMaterialGroup.FName");
//        request.setFieldKeys("FNumber,FNumber,FName,FUseOrgId,FUseOrgId.FNUMBER,FUseOrgId.FName, FCategoryID, FCategoryID.FNUMBER,FCategoryID.FName, FSpecification,"
//                + "FBaseUnitId.FNumber,FBaseUnitId.FName,FStoreUnitID.FNumber,FStoreUnitID.FName,FSalePriceUnitId.FNumber,FSalePriceUnitId.FName,FPurchasePriceUnitId.FNumber,FPurchasePriceUnitId.FName,FTaxRateId.FTaxRate,"
//                + "FMaterialGroup.FNumber,FIsInventory,FIsSale,FIsPurchase,FLENGTH,FVOLUME,FWIDTH,FHEIGHT,FOldNumber,F_tmt_Text,F_njdm_Text,F_njxh_Text,F_wjdm_Text,F_wjxh_Text,"
//                + "F_pjdm_Text,F_pjxh1_Text,F_pjcspdm_Text,F_pjxh2_Text");
        request.setFilterString(String.format(filterStr, orgNumber));
        request.setOrderString("FMATERIALID ASC");

        String[] fields = new String[]{"id","number","name","useOrgId","useOrgNumber","useOrgName","categoryId","categoryNumber","categoryName","specification"
                ,"baseUnitCode","baseUnitId","storeUnitCode","storeUnitId","salePriceUnitCode","salePriceUnitId","purchasePriceUnitCode","purchasePriceUnitId",
                "taxRateId","materialGroupNumber","isInventory","isSale","isPurchase"
                ,"length","volume","width","height","oldNumber","tmtText","njdmText","njxhText","wjdmText","wjxhText","pjdmText","pjxh1Text"
                ,"pjcspdmText","pjxh2Text","materialGroupName"};
        List<KingDeeMaterial> objects = k3Logic.allSync(request, fields, KingDeeMaterial.class);
        if(CollectionUtils.isEmpty(objects)){
            return;
        }
        for(KingDeeMaterial kingDeeMaterial : objects){
            kingDeeMaterial.setTaxRateId(kingDeeMaterial.getTaxRateId().divide(BigDecimal.valueOf(100)));
        }
        if (!org.springframework.util.CollectionUtils.isEmpty(objects)) {

            //1.先删掉已存在的
            List<String> numbers = objects.stream().map(KingDeeMaterial::getNumber).collect(Collectors.toList());
            kingDeeMaterialService.lambdaUpdate().in(KingDeeMaterial::getNumber,numbers).remove();
            //2.新增新的物料
            kingDeeMaterialService.saveOrUpdateBatch(objects);
            //3.//更新调价单/价格表的商品名称
            List<String> materialIds = objects.stream().map(KingDeeMaterial::getId).collect(Collectors.toList());
            k3Mapper.updateProductBillItem(materialIds);
            k3Mapper.updateProductPrice(materialIds);
        }
    }

}
