package com.zfire.mall.manager.logic.k3;


import cn.hutool.core.collection.CollectionUtil;
import com.zfire.mall.manager.bean.k3.RequestApiBean;
import com.zfire.mall.manager.constant.KingDeeCons;
import com.zfire.mall.manager.plus.entity.KingDeePurchaseOrder;
import com.zfire.mall.manager.plus.entity.KingDeePurchaseOrderItem;
import com.zfire.mall.manager.plus.service.KingDeePurchaseOrderItemService;
import com.zfire.mall.manager.plus.service.KingDeePurchaseOrderService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Objects;

@Service
@Slf4j
public class K3PurchaseOrderLogic {

    @Autowired
    K3Logic k3Logic;

    @Autowired
    KingDeePurchaseOrderService kingDeePurchaseOrderService;

    @Autowired
    KingDeePurchaseOrderItemService kingDeePurchaseOrderItemService;


    public void allSync(String orgId, String orgNumber, String startTime, String endTime) throws Exception {
        // 调用采购订单接口
        RequestApiBean request = k3Logic.getInstance(orgId, orgNumber, KingDeeCons.PURCHASE_ORDER, startTime, endTime);
        String filterStr = "FDocumentStatus = 'C' "+ k3Logic.supplySql("FApproveDate", request.getStartTime(), request.getEndTime());
        request.setFieldKeys("FID,FBillNo,FBusinessType,FDate,FBillTypeID,FAssignSupplierId.FName,FPurchaseDeptId.FName,FPurchaserId.FName,FPayConditionId.FName,FApproverId.FName,FCreatorId.FName," +
                "FChangerId.FName,FBillAllAmount,FBillAmount,FBillTaxAmount,FSettleCurrId.FName");
        request.setFilterString(String.format(filterStr, orgNumber));
        request.setOrderString("FID ASC");

        String[] fields = new String[]{"id","billNo","businessType","fDate","billType","assignSupplierId","purchaseDeptId","purchaserId","payConditionId","approverName","creatorName",
                "changerName","billAllAmount","billAmount","billTaxAmount","settleCurr"};
        List<KingDeePurchaseOrder> objects = k3Logic.allSync(request, fields, KingDeePurchaseOrder.class);
        if (!CollectionUtils.isEmpty(objects)) {
            kingDeePurchaseOrderService.saveOrUpdateBatch(objects);
        }
        allSyncOrder(orgId,orgNumber,startTime,endTime);
    }


    public void allSyncOrder(String orgId, String orgNumber, String startTime, String endTime) throws Exception {
        // 调用采购订单接口
        RequestApiBean request = k3Logic.getInstance(orgId, orgNumber, KingDeeCons.PURCHASE_ORDER, startTime, endTime);
        String filterStr = "FDocumentStatus = 'C' "+ k3Logic.supplySql("FApproveDate", request.getStartTime(), request.getEndTime());
        request.setFieldKeys("FPOOrderEntry_FEntryID,FID,FMaterialId,FMaterialId.FName,FMaterialId.FNumber,FUnitId.FName,FQty,FPrice,FEntryAmount,FEntryNote,FAllAmount ," +
                "FInvoiceQty,FInvoiceAmount,FModel,FEntryTaxAmount,FTaxPrice");
        request.setFilterString(String.format(filterStr, orgNumber));
        request.setOrderString("FID ASC");

        String[] fields = new String[]{"id","fId","materialId","materialName","materialNumber","unit","qty","price","entryAmount","entryNote","allAmount","invoiceQty",
                "invoiceAmount","model","entryTaxAmount","taxPrice"};
        List<KingDeePurchaseOrderItem> objects = k3Logic.allSync(request, fields, KingDeePurchaseOrderItem.class);
        if (CollectionUtil.isNotEmpty(objects)) {
            kingDeePurchaseOrderItemService.saveOrUpdateBatch(objects);
        }
    }


}
