package com.zfire.mall.manager.logic.k3;


import cn.hutool.core.collection.CollectionUtil;
import com.zfire.mall.manager.bean.k3.RequestApiBean;
import com.zfire.mall.manager.constant.KingDeeCons;
import com.zfire.mall.manager.plus.entity.KingDeePurchaseStockIn;
import com.zfire.mall.manager.plus.entity.KingDeePurchaseStockInItem;
import com.zfire.mall.manager.plus.service.KingDeePurchaseStockInItemService;
import com.zfire.mall.manager.plus.service.KingDeePurchaseStockInService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Objects;

@Service
@Slf4j
public class K3PurchaseStockInLogic {

    @Autowired
    K3Logic k3Logic;

    @Autowired
    KingDeePurchaseStockInService kingDeePurchaseStockInService;

    @Autowired
    KingDeePurchaseStockInItemService kingDeePurchaseStockInItemService;

    public List<KingDeePurchaseStockInItem> allSync(String orgId, String orgNumber, String startTime, String endTime) throws Exception {
        // 调用入库单接口
        RequestApiBean request = k3Logic.getInstance(orgId, orgNumber, KingDeeCons.PURCHASE_IN_STOCK, startTime, endTime);
        String filterStr = "FDocumentStatus = 'C' "+ k3Logic.supplySql("FApproveDate", request.getStartTime(), request.getEndTime());
        request.setFieldKeys("FID,FBillNo,FDate,FStockOrgId,FDemandOrgId,FPurchaseOrgId,FSupplierId,FStockDeptId,FApproverId.FName,FApproveDate,FCancellerId,FCancelDate," +
                "FSupplyId.FName,FDocumentStatus");
        request.setFilterString(String.format(filterStr, orgNumber));
        request.setOrderString("FID ASC");

        String[] fields = new String[]{"id","billNo","fDate","stockOrgId","demandOrgId","purchaseOrgId","supplierId","stockDeptId","approverId","approveDate","cancellerId","cancelDate",
                "supplyName,fdDocumentStatus"};
        request.setNotTime(true);
        List<KingDeePurchaseStockIn> objects = k3Logic.allSync(request, fields, KingDeePurchaseStockIn.class);
        if (!CollectionUtils.isEmpty(objects)) {
            kingDeePurchaseStockInService.saveOrUpdateBatch(objects);
        }

        return allSyncOrder(orgId,orgNumber,startTime,endTime);
    }


    public List<KingDeePurchaseStockInItem> allSyncOrder(String orgId, String orgNumber, String startTime, String endTime) throws Exception {
        // 调用入库单接口
        RequestApiBean request = k3Logic.getInstance(orgId, orgNumber, KingDeeCons.PURCHASE_IN_STOCK, startTime, endTime);
        String filterStr = "FDocumentStatus = 'C' "+ k3Logic.supplySql("FApproveDate", request.getStartTime(), request.getEndTime());
        request.setFieldKeys("FInStockEntry_FEntryID,FID,FBillNo,FMaterialId,FMaterialId.FNumber,FMaterialId.FName,FStockId,FNote,FMustQty,FRealQty,FUnitId.FName,FAuxUnitQty,FPrice,FEntryTaxRate" +
                ",FAmount,FTaxPrice,FEntryTaxAmount,FAllAmount,FUOM,FStockId.FName,F_cddh_Text");
        request.setFilterString(String.format(filterStr, orgNumber));
        request.setOrderString("FID ASC");

        String[] fields = new String[]{"id","fId","billNo","materialId","materialNumber","materialName","stockId","note","mustQty","realQty","unit","auxUnitQty","price","entryTaxRate",
                "amount","taxPrice","entryTaxAmount","allAmount","uom","stockName","cddhText"};
        List<KingDeePurchaseStockInItem> objects = k3Logic.allSync(request, fields, KingDeePurchaseStockInItem.class);
/*        if (CollectionUtil.isNotEmpty(objects)) {
            kingDeePurchaseStockInItemService.saveOrUpdateBatch(objects);
        }*/
        return objects;
    }


}
