package com.zfire.mall.manager.logic.k3;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.zfire.mall.manager.bean.k3.RequestApiBean;
import com.zfire.mall.manager.bean.wallet.CustomerWalletAmountBean;
import com.zfire.mall.manager.constant.Constant;
import com.zfire.mall.manager.constant.KingDeeCons;
import com.zfire.mall.manager.logic.wallet.WalletLogic;
import com.zfire.mall.manager.plus.entity.CustomerWallet;
import com.zfire.mall.manager.plus.entity.KingDeeReceipt;
import com.zfire.mall.manager.plus.service.KingDeeCustomerService;
import com.zfire.mall.manager.plus.service.KingDeeReceiptService;
import com.zfire.mall.manager.plus.service.KingDeeWalletService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
public class K3ReceiptLogic {

    @Autowired
    KingDeeReceiptService kingDeeReceiptService;
    @Autowired
    KingDeeCustomerService kingDeeCustomerService;
    @Autowired
    KingDeeWalletService kingDeeWalletService;
    @Autowired
    K3Logic k3Logic;
    @Autowired
    WalletLogic walletLogic;


    /**
     * 同步收款单-反审
     * @param orgId
     * @param orgNum
     * @param startTime
     * @param endTime
     * @throws Exception
     */
    @Transactional(propagation = Propagation.REQUIRES_NEW)
    public void syncSkdD(String orgId,String orgNum,String startTime,String endTime) throws Exception {
        String formId =  KingDeeCons.WALLET_SKD;
        //同步 反审的 收款单  formId永远查询所有
        RequestApiBean request = k3Logic.getInstance(orgId, orgNum, null, startTime, endTime);
        request.setFormId(formId);
        request.setNotTime(true);
        String filterStr = "FDocumentStatus ='D'";
        this.allSyncSkd(request,filterStr);
    }
    /**
     * 同步收款单-已审核
     * @param startTime
     * @param endTime
     */
    @Transactional(propagation = Propagation.REQUIRES_NEW)
    public void syncSkdC(String orgId,String orgNum,String startTime,String endTime) throws Exception {
        String formId =  KingDeeCons.WALLET_SKD;
        //同步已审核的收款单
        RequestApiBean request2 = k3Logic.getInstance(orgId, orgNum, formId, startTime, endTime);
        String filterStr2 = "FDocumentStatus ='C' "+k3Logic.supplySql("FApproveDate", request2.getStartTime(), request2.getEndTime());
        this.allSyncSkd(request2,filterStr2);
    }

    private void allSyncSkd(RequestApiBean request,String filterStr) throws Exception {

        //付款单位 ： FPAYUNIT
        // 同步钱包接口
        request.setFieldKeys("FRECEIVEBILLENTRY_FEntryID,FBillNo,FDOCUMENTSTATUS,FRECAMOUNTFOR_E,F_kls_Assistant1,FCONTACTUNIT,FREALRECAMOUNTFOR_D,FDATE," +
                "FPAYUNIT,FSETTLEDISTAMOUNTFOR,FSALEDEPTID,FSALEERID,FREMARK,FACCOUNTID," +
                "FSALEGROUPID,FSETTLETYPEID,FSETTLEORGID,FPAYORGID,FCURRENCYID,FPAYUNITTYPE,FApproveDate");
        request.setFilterString(String.format(filterStr));
        request.setOrderString("FID ASC");
        String[] fields = new String[]{"id","billNo","status","amount","walletId","customerId","hAmount","theTime",
                "wlCompanyType","discount","saleDepartment","salePerson","remark","bankAccount",
                "saleGroup","settleType","settleOrg","skzz","bibie","customerType","examineTime"};

        List<KingDeeReceipt> objects = k3Logic.allSync(request, fields, KingDeeReceipt.class);

        if (CollectionUtils.isEmpty(objects)) {
            return;
        }

        objects.forEach(v -> v.setBillType("1").setId(v.getBillType() + v.getId()));
        this.initAmount(objects);
        //处理金额充值-----------------结束
        kingDeeReceiptService.saveOrUpdateBatch(objects);
    }



    /**
     * 同步退款单-反审
     */
    @Transactional(propagation = Propagation.REQUIRES_NEW)
    public void syncTkdD(String orgId,String orgNum,String startTime,String endTime) throws Exception {
        String formId =  KingDeeCons.WALLET_TKD;
        //同步 反审的 退款单
        RequestApiBean request = k3Logic.getInstance(orgId, orgNum, null, startTime, endTime);
        request.setFormId(formId);
        request.setNotTime(true);
        String filterStr = "FDocumentStatus ='D'";
        this.allSyncTkd(request,filterStr);
    }
    /**
     * 同步退款单-已审核
     */
    public void syncTkdC(String orgId,String orgNum,String startTime,String endTime) throws Exception {
        String formId =  KingDeeCons.WALLET_TKD;
        //同步已审核的退款单
        RequestApiBean request2 = k3Logic.getInstance(orgId, orgNum, formId, startTime, endTime);
        String filterStr2 = "FDocumentStatus ='C' "+k3Logic.supplySql("FApproveDate", request2.getStartTime(), request2.getEndTime());
        this.allSyncTkd(request2,filterStr2);
    }


    private void allSyncTkd(RequestApiBean request,String filterStr) throws Exception {
        //收款单位：FRECTUNIT
        // 同步钱包接口
        request.setFieldKeys("FREFUNDBILLENTRY_FEntryID,FBillNo,FDOCUMENTSTATUS,FREALREFUNDAMOUNTFOR_D,F_kls_Assistant1, " +
                "FCONTACTUNIT,FREFUNDAMOUNTFOR_E,FDATE,FRECTUNIT, " +
                "FSALEDEPTID,FSALEERID,FREMARK,FACCOUNTID,FSALEGROUPID, " +
                "FSETTLETYPEID,FSETTLEORGID,FPAYORGID,FCURRENCYID,FRECTUNITTYPE,FApproveDate");
        request.setFilterString(String.format(filterStr));
        request.setOrderString("FID ASC");
        String[] fields = new String[]{"id","billNo","status","amount","walletId",
                "customerId","hAmount","theTime", "wlCompanyType","saleDepartment",
                "salePerson","remark","bankAccount", "saleGroup","settleType",
                "settleOrg","skzz","bibie","customerType","examineTime"};

        List<KingDeeReceipt> objects = k3Logic.allSync(request, fields, KingDeeReceipt.class);
        if (CollectionUtils.isEmpty(objects)) {
            return;
        }
        objects.forEach(v -> v.setBillType("2").setId(v.getBillType() + v.getId()));
        this.initAmount(objects);
        //处理金额充值-----------------结束
        kingDeeReceiptService.saveOrUpdateBatch(objects);
    }



    @Transactional(propagation = Propagation.REQUIRES_NEW)
    public void syncFkdD(String orgId,String orgNum,String startTime,String endTime) throws Exception {
        String formId =  KingDeeCons.WALLET_FKD;
        //同步 反审的 付款单
        RequestApiBean request = k3Logic.getInstance(orgId, orgNum, null, startTime, endTime);
        request.setFormId(formId);
        request.setNotTime(true);
        String filterStr = "FDocumentStatus ='D'";
        this.allSyncFkd(request,filterStr);
    }


    /**
     * 同步付款单-用来计算钱包余额
     * @param startTime
     * @param endTime
     */
    @Transactional(propagation = Propagation.REQUIRES_NEW)
    public void syncFkdC(String orgId,String orgNum,String startTime,String endTime) throws Exception {
        String formId =  KingDeeCons.WALLET_FKD;
        //同步已审核的付款单
        RequestApiBean request2 = k3Logic.getInstance(orgId, orgNum, formId, startTime, endTime);
        String filterStr2 = "FDocumentStatus ='C' "+k3Logic.supplySql("FApproveDate", request2.getStartTime(), request2.getEndTime());
        this.allSyncFkd(request2,filterStr2);
    }

    private void allSyncFkd(RequestApiBean request,String filterStr) throws Exception {
        // 同步钱包接口
        request.setFieldKeys("FPAYBILLENTRY_FEntryID,FBillNo,FDOCUMENTSTATUS,FREALPAYAMOUNTFOR_D,F_kls_Assistant1," +
                " FCONTACTUNIT,FREALPAYAMOUNTFOR_D,FDATE,FSETTLEDISTAMOUNTFOR," +
                " FSETTLETYPEID,FREMARK,FACCOUNTID, " +
                "FSETTLEORGID,FPAYORGID,FCURRENCYID,FOUTCONTACTTYPE,FApproveDate");
        request.setFilterString(String.format(filterStr));
        request.setOrderString("FID ASC");
        String[] fields = new String[]{"id","billNo","status","amount","walletId",
                "customerId","hAmount","theTime","discount",
                "settleType","remark","bankAccount",
                "settleOrg","skzz","bibie","customerType","examineTime"};

        List<KingDeeReceipt> objects = k3Logic.allSync(request, fields, KingDeeReceipt.class);
        if (CollectionUtils.isEmpty(objects)) {
            return;
        }
        objects.forEach(v -> v.setBillType("3").setId(v.getBillType() + v.getId()));
        this.initAmount(objects);
        //处理金额充值-----------------结束
        kingDeeReceiptService.saveOrUpdateBatch(objects);
    }





    /**
     * 现金钱包金额充值/反审
     * @param objects
     * @throws InterruptedException
     */
    public void initAmount(List<KingDeeReceipt> objects) throws InterruptedException {
        //一键生成经销商钱包
//        List<String> customerIds = objects.stream().filter(v -> v != null && StringUtils.isNotEmpty(v.getWalletId()))
//                .map(KingDeeReceipt::getCustomerId).distinct().collect(Collectors.toList());
//        for (String customerId : customerIds) {
//            walletLogic.initByK3Customer(customerId);
//        }
        walletLogic.initByK3Customer(null);
        log.info("收款单/退款单/付款单："+JSONObject.toJSONString(objects));

        List<CustomerWalletAmountBean> customerWalletAmountBeans = new ArrayList<>();
        //处理收款单/返利单逻辑/付款单
        for(KingDeeReceipt kingDeeReceipt : objects){
            if(kingDeeReceipt == null || (!kingDeeReceipt.getStatus().equals("C") && !kingDeeReceipt.getStatus().equals("D"))){
                continue;
            }
            if(StringUtils.isEmpty(kingDeeReceipt.getWalletId().trim()) || StringUtils.isEmpty(kingDeeReceipt.getCustomerId().trim())){
                continue;
            }

            //本次同步过来的钱
            BigDecimal amount = kingDeeReceipt.getAmount();
            if(kingDeeReceipt.getStatus().equals("D")) {
                amount = BigDecimal.valueOf(0);
            }

            //检查是否存在反审过来的单,如果之前存在审核通过的单，又有相同的单过来，代表该单这次被反审了，需要减掉它
            KingDeeReceipt oldBill = kingDeeReceiptService.getById(kingDeeReceipt.getId());
            if(oldBill != null && oldBill.getExamineTime() != null && kingDeeReceipt.getExamineTime() != null
                    && oldBill.getExamineTime().getTime() == kingDeeReceipt.getExamineTime().getTime()){
                log.info("【重复同步的收款单/付款单】id:"+kingDeeReceipt.getId());
                continue;
            }
            if(oldBill != null && oldBill.getStatus().equals("C")){
                BigDecimal subAmount = BigDecimal.valueOf(0).subtract(oldBill.getAmount());
                customerWalletAmountBeans.add(this.doAmount(oldBill,subAmount,oldBill.getRemark()));
            }

            customerWalletAmountBeans.add(this.doAmount(kingDeeReceipt,amount,kingDeeReceipt.getRemark()));
        }
        walletLogic.doWalletAmount(customerWalletAmountBeans);
    }


    private CustomerWalletAmountBean doAmount(KingDeeReceipt kingDeeReceipt,BigDecimal amountParam,String remark) throws InterruptedException {
        String customerId = kingDeeReceipt.getCustomerId();
        String walletId = kingDeeReceipt.getWalletId();
        String billType = kingDeeReceipt.getBillType();//1=收款单  2=退款单   3=付款单
        String billNo = kingDeeReceipt.getBillNo();

        //退款单 和 付款单 要反着来
        if("3".equals(billType) || "2".equals(billType)){
            amountParam = BigDecimal.valueOf(0).subtract(amountParam);
        }

        double amount = amountParam.stripTrailingZeros().doubleValue();
        if(amount == 0){
            return null;
        }
        if(StringUtils.isEmpty(customerId) || StringUtils.isEmpty(walletId)){
            return null;
        }
        CustomerWallet customerWallet = walletLogic.queryCustomerWallet(customerId, walletId);
        if(customerWallet == null){
            return null;
        }
        if(amount > 0) {
            String billName = "";
            if(billType.equals("1")){
                billName = Constant.BILL_TYPE_SKD;
            }else if(billType.equals("2")){
                billName = Constant.BILL_TYPE_TKD;
            }else{
                billName = Constant.BILL_TYPE_FKD;
            }
            return new CustomerWalletAmountBean(customerWallet.getCustomerWalletId(), amountParam, remark, billName, new Date(), billNo, 1, "add");
        } else{
            String billName = "";
            if(billType.equals("1")){
                billName = Constant.BILL_TYPE_SKD;
            }else if(billType.equals("2")){
                billName = Constant.BILL_TYPE_TKD;
            }else{
                billName = Constant.BILL_TYPE_FKD;
            }
            BigDecimal a = BigDecimal.valueOf(0).subtract(amountParam);
            return new CustomerWalletAmountBean(customerWallet.getCustomerWalletId(), a, remark, billName, new Date(),billNo , 1,"sub");
        }
    }


}
