package com.zfire.mall.manager.logic.k3;


import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.zfire.mall.manager.bean.k3.RequestApiBean;
import com.zfire.mall.manager.constant.KingDeeCons;
import com.zfire.mall.manager.plus.entity.KingDeeTransfer;
import com.zfire.mall.manager.plus.entity.KingDeeTransferItem;
import com.zfire.mall.manager.plus.service.KingDeeTransferItemService;
import com.zfire.mall.manager.plus.service.KingDeeTransferService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;

@Service
@Slf4j
public class K3TransferLogic {


    @Autowired
    K3Logic k3Logic;

    @Autowired
    KingDeeTransferService kingDeeTransferService;

    @Autowired
    KingDeeTransferItemService kingDeeTransferItemService;


    public void allSync(String orgId, String orgNumber, String startTime, String endTime) throws Exception {
        // 调用调拨单接口
        RequestApiBean request = k3Logic.getInstance(orgId, orgNumber, KingDeeCons.STOCK_TRANSFER, startTime, endTime);
        String filterStr = "FDocumentStatus = 'C' "+ k3Logic.supplySql("FApproveDate", request.getStartTime(), request.getEndTime());
        request.setFieldKeys("FID,FBillNo,FStockOrgId.FName,FStockOutOrgId.FName,FTransferBizType,FNote,FApproverId.FName,FCancellerId.FName,FBillTypeID.FName,FDocumentStatus,FDate,FApproveDate,FCancelDate");
        request.setFilterString(String.format(filterStr, orgNumber));
        request.setOrderString("FID ASC");

        String[] fields = new String[]{"id","billNo","stockOrgId","stockOutOrgId","transferBizType","note","approverId","cancellerId","billType","documentStatus","fDate","approveDate","cancelDate"};
        List<KingDeeTransfer> objects = k3Logic.allSync(request, fields, KingDeeTransfer.class);
        syncTransferItem(orgId,orgNumber,startTime,endTime);
        if (CollectionUtils.isNotEmpty(objects)) {
            kingDeeTransferService.saveOrUpdateBatch(objects);
        }

    }

    public void syncTransferItem(String orgId, String orgNumber, String startTime, String endTime) throws Exception {
        // 调用调拨单物料接口
        RequestApiBean request = k3Logic.getInstance(orgId, orgNumber, KingDeeCons.STOCK_TRANSFER, startTime, endTime);
        String filterStr = "FDocumentStatus = 'C' "+ k3Logic.supplySql("FApproveDate", request.getStartTime(), request.getEndTime());
        request.setFieldKeys("FBillEntry_FEntryID,FID,FBillNo,FMaterialId,FMaterialId.FNumber,FMaterialName,FSrcStockId.FName,FDestStockId.FName,FQty,FOwnerId,FOwnerOutId,FModel,FBaseUnitId.FName,FConsignAmount,FConsignPrice,FSrcStockLocId,FDestStockLocId,FAuxPropId");
        request.setFilterString(String.format(filterStr, orgNumber));
        request.setOrderString("FID ASC");

        String[] fields = new String[]{"id","fId","billNo","materialId","materialNumber","materialName","srcStockId","destStockId","qty","ownerId","ownerOutId","model","baseUnitId","consignAmount","consignPrice","srcStockLocId","destStockLocId","auxPropId"};
        List<KingDeeTransferItem> objects = k3Logic.allSync(request, fields, KingDeeTransferItem.class);
        if (CollectionUtil.isNotEmpty(objects)) {
            kingDeeTransferItemService.saveOrUpdateBatch(objects);
        }
    }

}
