package com.zfire.mall.manager.logic.wallet;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.zfire.mall.manager.bean.wallet.CustomerWalletAmountBean;
import com.zfire.mall.manager.commonmapper.wallet.WalletCommonMapper;
import com.zfire.mall.manager.constant.Constant;
import com.zfire.mall.manager.enums.WalletTypeEnum;
import com.zfire.mall.manager.exception.RemoteServiceException;
import com.zfire.mall.manager.logic.k3.K3CustomerLogic;
import com.zfire.mall.manager.plus.entity.CustomerWallet;
import com.zfire.mall.manager.plus.entity.CustomerWalletRecord;
import com.zfire.mall.manager.plus.service.*;
import lombok.extern.slf4j.Slf4j;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.integration.redis.util.RedisLockRegistry;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionSynchronization;
import org.springframework.transaction.support.TransactionSynchronizationManager;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.stream.Collectors;

@Slf4j
@Service
public class WalletLogic {

    @Autowired
    WalletRebateService walletRebateService;
    @Autowired
    WalletRebateSaleTypeService walletRebateSaleTypeService;
    @Autowired
    CustomerWalletService customerWalletService;
    @Autowired
    CustomerWalletRecordService customerWalletRecordService;
    @Autowired
    KingDeeWalletService kingDeeWalletService;
    @Autowired
    CustomerWalletTransferService customerWalletTransferService;
    @Autowired
    ReceivableOrderService receivableOrderService;
    @Autowired
    ReceivableOrderItemService receivableOrderItemService;
    @Autowired
    SaleTypeService saleTypeService;

    @Autowired
    RedisLockRegistry redisLockRegistry;
    @Autowired
    K3CustomerLogic k3CustomerLogic;
    @Autowired
    WalletLogic walletLogic;
    @Autowired
    KingDeeCustomerService kingDeeCustomerService;
    @Autowired
    WalletCommonMapper walletCommonMapper;


    final List<String> GLB_BillTypeList = new ArrayList<String>(){
        {
            add(Constant.BILL_TYPE_SKDFS);
            add(Constant.BILL_TYPE_TKDFS);
            add(Constant.BILL_TYPE_FKDFS);
            add(Constant.BILL_TYPE_OFKD);
            add(Constant.BILLTYPE_FLD_ZR);
        }
    };

    final List<String> GLB_BILL_SOURCE = new ArrayList<String>(){
        {
            add("工程价差");
            add("仓储费");
            add("工程收差");
        }
    };

    /**
     * 生成经销商的现金钱包和返利钱包
     * @param customerId
     */
    public void initByK3Customer(String customerId) throws InterruptedException {
//        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.LOCK_WALLET + customerId);
//        if(!obtain.tryLock(10,TimeUnit.SECONDS)){
//            throw new RemoteServiceException("系统繁忙，请稍后再试");
//        }
        try {
            walletCommonMapper.initCustomerWallet(customerId);
        }catch(Exception e){
            log.error("生成钱包失败");
        }
//        try {
//
//        } catch (Exception e) {
//            log.error("初始化经销商钱包失败",e);
//        } finally {
//            obtain.unlock();
//        }
    }



    /**
     * 批量修改钱包金额，同单据多个钱包的每个钱包只需要锁1次
     * @param list
     * @throws InterruptedException
     */
    public void doWalletAmount(List<CustomerWalletAmountBean> list) throws InterruptedException {
        if(CollectionUtils.isEmpty(list)){
            return;
        }
        list = list.stream().filter(v -> v != null && StringUtils.isNotEmpty(v.getCustomerWalletId())).collect(Collectors.toList());
        List<String> customerWalletIds = list.stream().filter(v -> v != null).map(CustomerWalletAmountBean::getCustomerWalletId).distinct().collect(Collectors.toList());
        log.info("【钱包金额处理】customerWalletIds:{}",JSONObject.toJSONString(customerWalletIds));
        for(String customerWalletId : customerWalletIds){
            Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.LOCK_WALLET + customerWalletId);
            if(!obtain.tryLock(10,TimeUnit.SECONDS)){
                throw new RemoteServiceException("钱包已冻结:"+customerWalletId);
            }
            try{
                List<CustomerWalletAmountBean> customerWalletAmountBeans = list.stream().filter(v -> v.getCustomerWalletId().equals(customerWalletId)).collect(Collectors.toList());
                for(CustomerWalletAmountBean bean : customerWalletAmountBeans) {
                    CustomerWallet customerWallet = walletCommonMapper.getCustomerWalletByIdForUpdate(customerWalletId);
                    bean.setCustomerWallet(customerWallet);
                    if(bean.getType().equals("add")) {
                        this.addWalletAmount(customerWallet, bean.getAmount(), bean.getRemark(), bean.getBillType(), bean.getTheTime(), bean.getBillNo(),
                                bean.getFlag(), bean.getAllowanceAmount(), bean.getWithholdAmount());
                    }else{
                        this.subWalletAmount(customerWallet,bean.getAmount(),bean.getRemark(),bean.getBillType(),bean.getTheTime(),bean.getBillNo(),
                                bean.getFlag(),bean.getAllowanceAmount(),bean.getWithholdAmount());
                    }
                }
            }finally {
                this.txCallUnlock(obtain);
                //obtain.unlock();
            }
        }
    }

    private void addWalletAmount(CustomerWallet customerWallet, BigDecimal amount,String remark,String billType,
                                Date theTime,String billNo,Integer flag,BigDecimal allowanceAmount,BigDecimal withholdAmount) throws InterruptedException {

        if(customerWallet == null){
            throw new RemoteServiceException("调取钱包失败，请稍后再试");
        }
        if(amount.doubleValue() < 0){
            throw new RemoteServiceException("钱包不可添加负数");
        }
        //现金钱包
        if(customerWallet.getType().equals(WalletTypeEnum.COMMONLY.toString())) {
            //1.先还信用额度
            BigDecimal sub = customerWallet.getUsedCreditAmount().subtract(amount);
            if (sub.doubleValue() > 0) {
                //已使用的信用额度 比 加的钱 大
                customerWallet.setUsedCreditAmount(sub);
                customerWallet.setFreeCreditAmount(customerWallet.getFreeCreditAmount().add(amount));
            } else {
                //已使用的信用额度 比 加的钱 小
                customerWallet.setFreeCreditAmount(customerWallet.getFreeCreditAmount().add(customerWallet.getUsedCreditAmount()));
                customerWallet.setAmount(customerWallet.getAmount().subtract(sub));//负负得正
                customerWallet.setUsedCreditAmount(BigDecimal.valueOf(0));
            }
            customerWallet.updateById();
        }else if(customerWallet.getType().equals(WalletTypeEnum.REBATE.toString())){
            //返利钱包
            customerWallet.setAmount(customerWallet.getAmount().add(amount));
            customerWallet.updateById();
        }
        //记录流水
        this.saveCustomerWalletRecord(customerWallet,amount,remark,billType,theTime,billNo,flag,"IN",allowanceAmount,withholdAmount,billNo);
    }


    /**
     * 钱包余额-扣除
     * @param amount 金额
     * @param remark 说明
     * @param billType 单据类型
     * @param theTime 单据日期
     * @param billNo 单据号
     * @param flag 1=预付货款 2=订单占款 3=支付货款 4=押金 5=工程保证金 6=其他暂扣款
     * @param allowanceAmount  折让金额
     * @param withholdAmount  暂扣返利
     * @throws InterruptedException
     */
    private void subWalletAmount(CustomerWallet customerWallet,BigDecimal amount,String remark,String billType,
                                Date theTime,String billNo,Integer flag,BigDecimal allowanceAmount,BigDecimal withholdAmount) throws InterruptedException {

            if(amount.doubleValue() < 0){
                throw new RemoteServiceException("钱包不可扣除负数");
            }
            if(customerWallet == null){
                throw new RemoteServiceException("调取钱包失败，请稍后再试");
            }
            //反审收款单和退款单，允许钱包负数
            if(! GLB_BillTypeList.contains(billType)) {
                if (customerWallet.getFreeCreditAmount().add(customerWallet.getAmount()).doubleValue() < amount.doubleValue()) {
                    //返利钱包可能是负数，如果此时需要扣返利金额0元，则不需要拦截
                    if((customerWallet.getType().equals(WalletTypeEnum.REBATE.toString()) && amount.doubleValue() != 0)
                            ||  customerWallet.getType().equals(WalletTypeEnum.COMMONLY.toString())) {
                        throw new RemoteServiceException("操作失败，钱包:【" + customerWallet.getCustomerWalletName() + "】可使用金额不足");
                    }
                }
            }

            //处理钱包逻辑
            //现金钱包
            if(customerWallet.getType().equals(WalletTypeEnum.COMMONLY.toString())) {
                if (customerWallet.getAmount().subtract(amount).doubleValue() > 0) {
                    customerWallet.setAmount(customerWallet.getAmount().subtract(amount));
                } else {
                    BigDecimal sub = customerWallet.getAmount().subtract(amount);
                    customerWallet.setAmount(sub);
                }
                customerWallet.updateById();
            }else if(customerWallet.getType().equals(WalletTypeEnum.REBATE.toString())){
                //返利钱包
                if(!Constant.BILLTYPE_FLD_ZR.equals(billType)){
                    if(customerWallet.getAmount().doubleValue() < amount.doubleValue() && amount.doubleValue() != 0){
                        throw new RemoteServiceException("返利钱包可扣除金额不足");
                    }
                }

                customerWallet.setAmount(customerWallet.getAmount().subtract(amount));
                customerWallet.updateById();
            }
            //记录流水
            this.saveCustomerWalletRecord(customerWallet,amount,remark,billType,theTime,billNo,flag,"OUT",allowanceAmount,withholdAmount,null);
    }



    public void txCallUnlock(Lock obtain){
        //事务提交完成后才释放锁,此方法禁止执行mysql相关操作,否则将导致重大问题。
        if(TransactionSynchronizationManager.isActualTransactionActive()) {
            TransactionSynchronizationManager.registerSynchronization(new TransactionSynchronization() {
                @Override
                public void afterCompletion(int status) {

                    if(TransactionSynchronization.STATUS_COMMITTED == status){
                        log.info("=========【钱包】======事务提交==============");
                    }else if(TransactionSynchronization.STATUS_ROLLED_BACK == status){
                        log.info("========【钱包】=========事务回滚============");
                    }

                    obtain.unlock();
                }
            });
        }else{
            log.info("【钱包】====没有事务===释放锁");
            obtain.unlock();
        }
    }


//    @Test
//    public void test(){
//        BigDecimal amount = new BigDecimal(4728);
//        CustomerWallet customerWallet = new CustomerWallet();
//        customerWallet.setAmount(BigDecimal.valueOf(4727));
//        customerWallet.setFreeCreditAmount(BigDecimal.ZERO);
//        customerWallet.setUsedCreditAmount(BigDecimal.ZERO);
//
//        //此处处理  钱包余额 - 要扣的钱 小于0的情况
//        //钱包余额差的钱（sub绝对为负数）
//        BigDecimal sub = customerWallet.getAmount().subtract(amount);
//
//        //用信用额度补足,如果信用额度也不够扣，则继续扣负现金钱包的余额,此处相加是因为sub绝对为负数
//        customerWallet.setAmount(sub);
//
//        log.info(JSONObject.toJSONString(customerWallet));
//    }


    /**
     * 台账流水
     * @param customerWallet 钱包
     * @param amount 金额
     * @param remark 摘要
     * @param billType 单据类型
     * @param theTime 单据日期
     * @param billNo 单据号
     * @param flag 1=预付货款 2=订单占款 3=支付货款 4=押金 5=工程保证金 6=其他暂扣款
     * @param amountType IN = 入账  OUT = 出账'
     * @param allowanceAmount 折让金额
     * @param withholdAmount 暂扣返利
     */
    public void saveCustomerWalletRecord(CustomerWallet customerWallet,BigDecimal amount,String remark,String billType,
                                          Date theTime,String billNo,Integer flag,String amountType,BigDecimal allowanceAmount,BigDecimal withholdAmount,String k3ReceiptId){
        //记录流水
        CustomerWalletRecord customerWalletRecord = new CustomerWalletRecord();
        customerWalletRecord.setCustomerWalletId(customerWallet.getCustomerWalletId());
        customerWalletRecord.setWalletName(customerWallet.getCustomerWalletName());
        customerWalletRecord.setType(customerWallet.getType());
        customerWalletRecord.setBillType(billType);
        customerWalletRecord.setTheTime(theTime);
        customerWalletRecord.setBillNo(billNo);
        customerWalletRecord.setCustomerId(customerWallet.getCustomerId());
        customerWalletRecord.setCustomerCode(customerWallet.getCustomerNumber());
        customerWalletRecord.setCustomerName(customerWallet.getCustomerName());
        customerWalletRecord.setAmountType(amountType);
        customerWalletRecord.setLastAmount(customerWallet.getAmount());
        customerWalletRecord.setAmount(amount);
        customerWalletRecord.setBalanceAmount(customerWallet.getAmount().subtract(customerWallet.getUsedCreditAmount()));
        customerWalletRecord.setRemark(remark);
        customerWalletRecord.setWalletRebateId(customerWallet.getWalletRebateId());
        customerWalletRecord.setMonth(new SimpleDateFormat("yyyy-MM").format(new Date()));
        customerWalletRecord.setFlag(flag);
        if(null != allowanceAmount){
            customerWalletRecord.setAllowanceAmount(allowanceAmount);
        }
        if(null != withholdAmount){
            customerWalletRecord.setWithholdAmount(withholdAmount);
        }
        if(StringUtils.isNotEmpty(k3ReceiptId)){
            customerWalletRecord.setK3ReceiptId(k3ReceiptId);
        }
        //如果是返利单 都记录到收入
        if(null != billType && Constant.BILLTYPE_FLD_ZR.equals(billType)){
            customerWalletRecord.setAmountType("IN");
            customerWalletRecord.setAmount(BigDecimal.valueOf(0).subtract(amount));
        }
        customerWalletRecord.insert();
    }


    public CustomerWallet queryCustomerWallet(String customerId,String walletId){
        CustomerWallet one = customerWalletService.lambdaQuery()
                .eq(CustomerWallet::getCustomerId, customerId)
                .eq(CustomerWallet::getWalletRebateId, walletId).one();
        return one;
    }

}
