package com.zfire.mall.manager.schedule;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.convert.Convert;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.zfire.mall.manager.bean.front.CallBackItemDTO;
import com.zfire.mall.manager.bean.front.CallBackItemList;
import com.zfire.mall.manager.bean.front.CostTicketVO;
import com.zfire.mall.manager.commonmapper.CustomerFrontOrderMapper;
import com.zfire.mall.manager.constant.Constant;
import com.zfire.mall.manager.exception.RemoteServiceException;
import com.zfire.mall.manager.plus.entity.*;
import com.zfire.mall.manager.plus.service.*;
import com.zfire.mall.manager.utils.JiaXianUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.integration.redis.util.RedisLockRegistry;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;


/**
 * 更新库存
 *
 * @author change
 */
@Component
@Slf4j
public class JiaXianSyncSchedule {

    @Autowired
    CustomerOrderService customerOrderService;

    @Autowired
    CustomerOrderItemService customerOrderItemService;


    @Autowired
    OrderTrackTocService orderTrackTocService;


    @Autowired
    RedisLockRegistry redisLockRegistry;

    @Autowired
    JiaXianUtils jiaXianUtils;


    @Autowired
    CostBillService costBillService;

    @Autowired
    CustomerCodeService customerCodeService;

    @Autowired
    CostMerchantService costMerchantService;

    @Autowired
    CostSalesService costSalesService;

    @Autowired
    CostTicketService costTicketService;

    @Autowired
    CostTicketBillService costTicketBillService;

    @Autowired
    CustomerFrontOrderMapper customerFrontOrderMapper;

    /**
     * 30秒toc物流
     */
    @Scheduled(fixedDelay = 60 * 1000)
    public void automaticCustomerOrder() throws Exception {



            List<OrderTrackToc> branchTrack = jiaXianUtils.getBranchTrack();

            if (CollectionUtils.isEmpty(branchTrack))
                return;

            orderTrackTocService.saveOrUpdateBatch(branchTrack);
            CallBackItemDTO callBackItemDTO = new CallBackItemDTO();
            callBackItemDTO.setCallBackType(Constant.JIAXIAN_BRANCH_ORDER_TRACK);

            List<CallBackItemList> callBackItemLists = new ArrayList<>();
            for (OrderTrackToc orderTrackToc : branchTrack) {
                CallBackItemList callBackItemList = new CallBackItemList();
                callBackItemList.setItemId(Convert.toInt(orderTrackToc.getOrderTrackId()));
                callBackItemList.setStatus("OK");
                callBackItemLists.add(callBackItemList);
            }
            callBackItemDTO.setCallBackItemList(callBackItemLists);
            jiaXianUtils.callBackSyncStatus(callBackItemDTO);

            customerFrontOrderMapper.updateToc();
            customerFrontOrderMapper.updateReturnToc();



    }


    /**
     * 30秒账单同步
     */
    @Scheduled(fixedDelay = 60 * 1000)
    public void automaticCostBill() throws Exception {



            List<CostBill> costBill = jiaXianUtils.getCostBill();

            if (CollectionUtils.isEmpty(costBill))
                return;

            costBillService.saveOrUpdateBatch(costBill);
            CallBackItemDTO callBackItemDTO = new CallBackItemDTO();
            callBackItemDTO.setCallBackType(Constant.JIAXIAN_COST_BILL);

            List<CallBackItemList> callBackItemLists = new ArrayList<>();
            for (CostBill orderTrackToc : costBill) {
                CallBackItemList callBackItemList = new CallBackItemList();
                callBackItemList.setItemId(Convert.toInt(orderTrackToc.getId()));
                callBackItemList.setStatus("OK");
                callBackItemLists.add(callBackItemList);
            }
            callBackItemDTO.setCallBackItemList(callBackItemLists);
            jiaXianUtils.callBackSyncStatus(callBackItemDTO);




    }


    /**
     * 30秒条码同步
     */
    @Scheduled(fixedDelay = 60 * 1000)
    public void automaticGetWMSScanBarCode() throws Exception {



            List<CustomerCode> wmsScanBarCode = jiaXianUtils.getWMSScanBarCode();

            if (CollectionUtils.isEmpty(wmsScanBarCode))
                return;

            customerCodeService.saveOrUpdateBatch(wmsScanBarCode);

            CallBackItemDTO callBackItemDTO = new CallBackItemDTO();
            callBackItemDTO.setCallBackType(Constant.JIAXIAN_OUT_SCAN_BAR_CODE);

            List<CallBackItemList> callBackItemLists = new ArrayList<>();
            for (CustomerCode customerCode : wmsScanBarCode) {
                CallBackItemList callBackItemList = new CallBackItemList();
                callBackItemList.setItemId(Convert.toInt(customerCode.getId()));
                callBackItemList.setStatus("OK");
                callBackItemLists.add(callBackItemList);
            }
            callBackItemDTO.setCallBackItemList(callBackItemLists);
            jiaXianUtils.callBackSyncStatus(callBackItemDTO);




    }


    /**
     *60秒同步销司对账单
     */
    @Scheduled(fixedDelay = 60 * 1000)
    public void automaticGetSyncCostSales() throws Exception {



            List<CostSales> syncCostSales = jiaXianUtils.getSyncCostSales();

            if (CollectionUtils.isEmpty(syncCostSales))
                return;

            costSalesService.saveOrUpdateBatch(syncCostSales);

            CallBackItemDTO callBackItemDTO = new CallBackItemDTO();
            callBackItemDTO.setCallBackType(Constant.JIAXIAN_COST_SALES);

            List<CallBackItemList> callBackItemLists = new ArrayList<>();
            for (CostSales customerCode : syncCostSales) {
                CallBackItemList callBackItemList = new CallBackItemList();
                callBackItemList.setItemId(Convert.toInt(customerCode.getId()));
                callBackItemList.setStatus("OK");
                callBackItemLists.add(callBackItemList);
            }
            callBackItemDTO.setCallBackItemList(callBackItemLists);
            jiaXianUtils.callBackSyncStatus(callBackItemDTO);



    }


    /**
     *60秒同步商户账单
     */
    @Scheduled(fixedDelay = 60 * 1000)
    public void automaticGetSyncCostMerchant() throws Exception {



            List<CostMerchant> syncCostMerchant = jiaXianUtils.getSyncCostMerchant();

            if (CollectionUtils.isEmpty(syncCostMerchant))
                return;

            costMerchantService.saveOrUpdateBatch(syncCostMerchant);

            CallBackItemDTO callBackItemDTO = new CallBackItemDTO();
            callBackItemDTO.setCallBackType(Constant.JIAXIAN_COST_MERCHANT);

            List<CallBackItemList> callBackItemLists = new ArrayList<>();
            for (CostMerchant customerCode : syncCostMerchant) {
                CallBackItemList callBackItemList = new CallBackItemList();
                callBackItemList.setItemId(Convert.toInt(customerCode.getId()));
                callBackItemList.setStatus("OK");
                callBackItemLists.add(callBackItemList);
            }
            callBackItemDTO.setCallBackItemList(callBackItemLists);
            jiaXianUtils.callBackSyncStatus(callBackItemDTO);


    }


    /**
     * 30秒条码同步
     */
    @Scheduled(fixedDelay = 60 * 1000)
    public void automaticGetDaCangWMSScanBarCode() throws Exception {



            List<CustomerCode> wmsScanBarCode = jiaXianUtils.getDaCangWMSScanBarCode();

            if (CollectionUtils.isEmpty(wmsScanBarCode))
                return;

            customerCodeService.saveOrUpdateBatch(wmsScanBarCode);

            CallBackItemDTO callBackItemDTO = new CallBackItemDTO();
            callBackItemDTO.setCallBackType(Constant.JIAXIAN_OUT_SCAN_BAR_CODE);

            List<CallBackItemList> callBackItemLists = new ArrayList<>();
            for (CustomerCode customerCode : wmsScanBarCode) {
                CallBackItemList callBackItemList = new CallBackItemList();
                callBackItemList.setItemId(Convert.toInt(customerCode.getId()));
                callBackItemList.setStatus("OK");
                callBackItemLists.add(callBackItemList);
            }
            callBackItemDTO.setCallBackItemList(callBackItemLists);
            jiaXianUtils.callBackDaCangSyncStatus(callBackItemDTO);




    }


    /**
     * 60秒同步异常工单
     */
    @Scheduled(fixedDelay = 60 * 1000)
    public void automaticGetSyncCostTicket() throws Exception {



            List<CostTicketVO> syncCostTicket = jiaXianUtils.getSyncCostTicket();

            if (CollectionUtils.isEmpty(syncCostTicket))
                return;

            List<CostTicket> costTickets = BeanUtil.copyToList(syncCostTicket, CostTicket.class);

            costTicketService.saveOrUpdateBatch(costTickets);

            for (CostTicketVO costTicketVO : syncCostTicket) {
                List<CostTicketBill> costTicketBills = BeanUtil.copyToList(costTicketVO.getCostTicketDetailList(), CostTicketBill.class);
                if (!CollectionUtils.isEmpty(costTicketBills))
                    costTicketBillService.saveOrUpdateBatch(costTicketBills);
            }

            CallBackItemDTO callBackItemDTO = new CallBackItemDTO();
            callBackItemDTO.setCallBackType(Constant.JIAXIAN_COST_TICKET);

            List<CallBackItemList> callBackItemLists = new ArrayList<>();
            for (CostTicket orderTrackToc : costTickets) {
                CallBackItemList callBackItemList = new CallBackItemList();
                callBackItemList.setItemId(Convert.toInt(orderTrackToc.getId()));
                callBackItemList.setStatus("OK");
                callBackItemLists.add(callBackItemList);
            }
            callBackItemDTO.setCallBackItemList(callBackItemLists);
            jiaXianUtils.callBackSyncStatus(callBackItemDTO);



    }

}
