package com.zfire.mall.manager.schedule;

import cn.hutool.core.date.DateUtil;
import com.zfire.mall.manager.enums.ExamineStatusEnum;
import com.zfire.mall.manager.enums.OrderTypeEnum;
import com.zfire.mall.manager.plus.entity.*;
import com.zfire.mall.manager.plus.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 定时清理保存状态的单(家用工程除外)
 */
@Component
public class OrderCleaSchedule {

    @Autowired
    RetailOrderService retailOrderService;
    @Autowired
    RetailOrderItemService retailOrderItemService;
    @Autowired
    EnginOrderService enginOrderService;
    @Autowired
    EnginOrderItemService enginOrderItemService;

    @Autowired
    RebateOrderItemService rebateOrderItemService;


    @Scheduled(cron = "0 0 0 * * ? ")
    public void task(){
        List<RetailOrder> list = retailOrderService.lambdaQuery().eq(RetailOrder::getExamineStatus, ExamineStatusEnum.SAVE.toString()).list();
        List<String> retailOrderIds = list.stream().map(RetailOrder::getId).collect(Collectors.toList());

//        List<EnginOrder> enginList = enginOrderService.lambdaQuery().eq(EnginOrder::getExamineStatus, ExamineStatusEnum.SAVE.toString())
//                .eq(EnginOrder::getEnginOrderType, OrderTypeEnum.TRADE.toString()).list();
//        List<String> enginOrderNos = enginList.stream().map(EnginOrder::getEnginOrderNo).collect(Collectors.toList());

        if(retailOrderIds.size() > 0) {
            //零售单主表
            retailOrderService.lambdaUpdate().in(RetailOrder::getId, retailOrderIds).remove();
            //零售单从表
            retailOrderItemService.lambdaUpdate().in(RetailOrderItem::getRetailOrderId, retailOrderIds).remove();
        }

        //update by ljh 2022-11-15  原因：引用的信息单数量会和订单对不上，因为被驳回的订单可能被晚上清理了
//        if(enginOrderNos.size() > 0) {
//            //商用工程单主表
//            enginOrderService.lambdaUpdate().in(EnginOrder::getEnginOrderNo, enginOrderNos).remove();
//            //商用工程单从表
//            enginOrderItemService.lambdaUpdate().in(EnginOrderItem::getEnginOrderNo,enginOrderNos).remove();
//        }

    }


    /**
     * 定时确认销售返利单(初审通过的单，30天还没确认，系统帮他确认)
     */
    @Scheduled(cron = "0 0 0 * * ? ")
    public void ackRebateOrder(){

        List<RebateOrderItem> list = rebateOrderItemService.lambdaQuery()
                .eq(RebateOrderItem::getExamineStatus,ExamineStatusEnum.OK_ONE.toString())
                .eq(RebateOrderItem::getCustomerIsConfirm, false)
                .le(RebateOrderItem::getCreateTime, DateUtil.offsetDay(new Date(), -30)).list();

        for(RebateOrderItem rebateOrderItem : list){
            rebateOrderItem.setCustomerIsConfirm(true);
            rebateOrderItem.setCustomerConfirmTime(new Date());
            rebateOrderItem.setUpdateTime(new Date());
            rebateOrderItem.setUpdateBy("system");
            rebateOrderItem.setRebateAmount(rebateOrderItem.getAmount());
        }
        rebateOrderItemService.updateBatchById(list);

    }



}
