package com.zfire.mall.manager.schedule;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.zfire.mall.manager.bean.front.CallBackItemDTO;
import com.zfire.mall.manager.bean.front.CallBackItemList;
import com.zfire.mall.manager.bean.front.CostTicketVO;
import com.zfire.mall.manager.bean.front.WorkerOrderInfo;
import com.zfire.mall.manager.bean.k3.RequestApiBean;
import com.zfire.mall.manager.constant.Constant;
import com.zfire.mall.manager.constant.KingDeeCons;
import com.zfire.mall.manager.exception.RemoteServiceException;
import com.zfire.mall.manager.logic.k3.K3Logic;
import com.zfire.mall.manager.logic.k3.K3PurchaseOrderLogic;
import com.zfire.mall.manager.logic.k3.K3PurchaseStockInLogic;
import com.zfire.mall.manager.plus.entity.*;
import com.zfire.mall.manager.plus.service.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.integration.redis.util.RedisLockRegistry;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.stream.Collectors;


/**
 * 更新库存
 *
 * @author change
 */
@Component
@Slf4j
public class SalesOrderSchedule{

    //预留单
    @Value("${supply.automatic.sales}")
    private String automaticSalesUrl;

    @Autowired
    StockCustomerService stockCustomerService;
    @Autowired
    StockService stockService;
    @Autowired
    KingDeeOrganizationService kingDeeOrganizationService;
    @Autowired
    K3Logic k3Logic;

    @Autowired
    ReceiptInvoiceAddedService receiptInvoiceAddedService;

    @Autowired
    ReceiptInvoiceOrdinaryService receiptInvoiceOrdinaryService;

    @Autowired
    SalesOrderService salesOrderService;

    @Autowired
    KingDeePurchaseStockInItemService kingDeePurchaseStockInItemService;


    @Autowired
    K3PurchaseOrderLogic k3PurchaseOrderLogic;
    @Autowired
    K3PurchaseStockInLogic k3PurchaseStockInLogic;

    @Autowired
    RedisLockRegistry redisLockRegistry;

    @Autowired
    KingDeePurchaseStockInService kingDeePurchaseStockInService;

    @Autowired
    KingDeePurchaseStockOutService kingDeePurchaseStockOutService;

    @Autowired
    KingDeePurchaseStockOutItemService kingDeePurchaseStockOutItemService;

    @Autowired
    KingDeeOtherStockInService kingDeeOtherStockInService;

    @Autowired
    KingDeeOtherStockInItemService kingDeeOtherStockInItemService;

    @Autowired
    KingDeeOtherStockOutService kingDeeOtherStockOutService;

    @Autowired
    KingDeeOtherStockOutItemService kingDeeOtherStockOutItemService;

    @Autowired
    OrderTrackService orderTrackService;

    @Autowired
    KingDeeInterfaceService kingDeeInterfaceService;

    @Autowired
    GongdanLogic gongdanLogic;

    @Autowired
    CustomerOrderService customerOrderService;

    @Autowired
    CustomerOrderItemService customerOrderItemService;


    @Autowired
    CustomerCodeService customerCodeService;

    @Autowired
    CustomerCodeInstallService customerCodeInstallService;


    @Autowired
    CustomerOrderInstallService customerOrderInstallService;

    @Autowired
    CustomerOrderInstallLogService customerOrderInstallLogService;


    //预留单
    @Value("${supply.automatic.stock.in}")
    private String automaticPuUrl;



    final static String orgId = "1";
    final static String orgNum = "100";



    /**
     * 同步销售出库单
     */
    @Scheduled(fixedDelay = 30 * 1000)
    public void automaticReserved() throws Exception {

        HttpUtil.get(automaticSalesUrl);
    }



    /**
     * 同步采购入库单
     */
    @Scheduled(fixedDelay = 30 * 1000)
    public void automatic() throws Exception {

        HttpUtil.get(automaticPuUrl);
    }


    /**
     * 同步物流消息到发货单
     */
    @Scheduled(fixedDelay = 60 * 1000)
    public void automaticOrderTrack() throws Exception {
        KingDeeInterface kingDeeInterface = kingDeeInterfaceService.lambdaQuery().eq(KingDeeInterface::getFormId, "ORDER_TRACK").last("limit 1").one();

        //DateTime newDate3 = DateUtil.offsetHour(DateUtil.parse(kingDeeInterface.getSyncLastTime(), "yyyy-MM-dd HH:mm:ss"), -1);

        List<OrderTrack> list = orderTrackService.lambdaQuery().ge(OrderTrack::getUpdateTime, kingDeeInterface.getSyncLastTime())
                .orderByAsc(OrderTrack::getTime)
                .list();

        if(CollectionUtils.isEmpty(list))
            return;

        kingDeeInterface.setSyncLastTime(DateUtil.formatDateTime(new Date()));
        kingDeeInterface.updateById();

        if (CollectionUtils.isNotEmpty(list)) {
            for (OrderTrack orderTrack : list) {
                InvoiceApplication invoiceApplication = new InvoiceApplication();
                invoiceApplication.setId(orderTrack.getOrderId());
                invoiceApplication.setOrderTrackStatus(orderTrack.getStatus());
                invoiceApplication.setOrderTrackType("YES");
                invoiceApplication.setOrderTrackTime(orderTrack.getTime());
                invoiceApplication.updateById();
            }
        }

    }




    /**
     * 60秒同步工单
     */
    @Scheduled(fixedDelay = 200 * 1000)
    public void automaticGetSyncCostTicket() throws Exception {
        List<CustomerOrder> list = customerOrderService.lambdaQuery()
                .isNotNull(CustomerOrder::getGongDanId)
                .ge(CustomerOrder::getCreateTime,DateUtil.offsetDay(new Date(),-1))
                .and(item -> item.isNotNull(CustomerOrder::getOrderState)
                .or().notIn(CustomerOrder::getOrderState, "10","9","99")).list();

        if (CollectionUtils.isEmpty(list))
            return;


        List<CustomerOrder> customerOrders = new ArrayList<>();
        List<CustomerOrderInstall> customerOrderInstalls = new ArrayList<>();
        List<CustomerOrderInstallLog> customerOrderInstallLogs = new ArrayList<>();
        List<CustomerCodeInstall> customerCodeInstalls = new ArrayList<>();
        for (CustomerOrder customerOrder : list) {

            List<CustomerOrderItem> customerOrderItems = customerOrderItemService.lambdaQuery().eq(CustomerOrderItem::getOrderId, customerOrder.getId()).list();
            List<WorkerOrderInfo> installOrderList = gongdanLogic.getInstallOrderList(customerOrder.getId());
            //赋值列表
            if (CollectionUtils.isEmpty(installOrderList))
                continue;
            WorkerOrderInfo workerOrderInfo = installOrderList.get(0);
            CustomerOrder customerOrderUp = new CustomerOrder();
            customerOrderUp.setPgid(workerOrderInfo.getPgid());
            customerOrderUp.setId(customerOrder.getId());
            customerOrderUp.setSalesNumber(workerOrderInfo.getSalesNumber());
            customerOrderUp.setSalesWname(workerOrderInfo.getSalesWname());
            customerOrderUp.setInstallName(workerOrderInfo.getInstallName());
            customerOrderUp.setInstallNumber(workerOrderInfo.getInstallNumber());
            customerOrderUp.setOrderState(workerOrderInfo.getOrderStateInt());
            customerOrderUp.setAppointState(workerOrderInfo.getAppointState());
            customerOrderUp.setWorkerName(workerOrderInfo.getWorkerName());
            customerOrderUp.setWorkerNumber(workerOrderInfo.getWorkerNumber());
            customerOrderUp.setWorkerPhone(workerOrderInfo.getWorkerPhone());
            customerOrderUp.setAdminOfficerName(workerOrderInfo.getAdminOfficerName());
            customerOrderUp.setAdminOfficerNumber(workerOrderInfo.getAdminOfficerNumber());
            customerOrders.add(customerOrderUp);


            //更新安装工单信息
            CustomerOrderInstall customerOrderInstall = new CustomerOrderInstall();
            customerOrderInstall.setGongDanId(customerOrder.getGongDanId());
            customerOrderInstall.setPgid(customerOrder.getPgid());
            customerOrderInstall.setSalesNumber(workerOrderInfo.getSalesNumber());
            customerOrderInstall.setSalesWname(workerOrderInfo.getSalesWname());
            customerOrderInstall.setSalesNumber(workerOrderInfo.getSalesNumber());
            customerOrderInstall.setSalesWname(workerOrderInfo.getSalesWname());
            customerOrderInstall.setInstallName(workerOrderInfo.getInstallName());
            customerOrderInstall.setInstallNumber(workerOrderInfo.getInstallNumber());

            customerOrderInstall.setOrderState(workerOrderInfo.getOrderStateInt());
            customerOrderInstall.setAppointState(workerOrderInfo.getAppointState());
            customerOrderInstall.setWorkerName(workerOrderInfo.getWorkerName());
            customerOrderInstall.setWorkerNumber(workerOrderInfo.getWorkerName());
            customerOrderInstall.setWorkerPhone(workerOrderInfo.getWorkerPhone());
            customerOrderInstall.setAdminOfficerName(workerOrderInfo.getAdminOfficerName());
            customerOrderInstall.setAdminOfficerNumber(workerOrderInfo.getAdminOfficerNumber());
            customerOrderInstall.setAfterNumber(workerOrderInfo.getAfterNumber());
            customerOrderInstall.setAfterWname(workerOrderInfo.getAfterWname());
            customerOrderInstall.setAfterTime(StringUtils.isEmpty(workerOrderInfo.getAfterTime())
                    ?null:DateUtil.date(Convert.toLong(workerOrderInfo.getAfterTime()+"000")));
            customerOrderInstall.setCreateNumber(workerOrderInfo.getCreateNumber());
            customerOrderInstall.setCreateWname(workerOrderInfo.getCreateWname());
            customerOrderInstall.setAssistName(workerOrderInfo.getAssistName());
            customerOrderInstall.setAssistNumber(workerOrderInfo.getAssistNumber());
            customerOrderInstall.setAssistPhone(workerOrderInfo.getAssistPhone());
            if (workerOrderInfo.getMachineNum() != null && workerOrderInfo.getNeedNum() != null)
                customerOrderInstall.setOrderOver((workerOrderInfo.getMachineNum()-workerOrderInfo.getNeedNum())+"/"+workerOrderInfo.getMachineNum());

            customerOrderInstall.setCreateTime(StringUtils.isEmpty(workerOrderInfo.getCreatetime())
                    ?null:DateUtil.date(Convert.toLong(workerOrderInfo.getCreatetime()+"000")));
            customerOrderInstalls.add(customerOrderInstall);

            //更新操作日志
            for (WorkerOrderInfo.Fkxx fkxx : workerOrderInfo.getFkxxs()) {
                CustomerOrderInstallLog customerOrderInstallLog = new CustomerOrderInstallLog();
                customerOrderInstallLog.setContent(fkxx.getOperatorContent());
                customerOrderInstallLog.setId(Convert.toStr(fkxx.getId()));
                customerOrderInstallLog.setType(fkxx.getOperatorType());
                customerOrderInstallLog.setUpdatetime(StringUtils.isEmpty(fkxx.getLastOperatorTime())
                        ?null:DateUtil.date(Convert.toLong(fkxx.getLastOperatorTime()+"000")));
                customerOrderInstallLog.setOperator(fkxx.getOperatorName());
                customerOrderInstallLog.setOperatorAccount(fkxx.getOperatorWebsit());
                customerOrderInstallLog.setWorkerOrderNo(customerOrderInstall.getGongDanId());
                customerOrderInstallLogs.add(customerOrderInstallLog);
            }


            //更新条码
            if (!CollectionUtils.isEmpty(workerOrderInfo.getOrderDetails())) {
                for (WorkerOrderInfo.OrderDetail orderDetail : workerOrderInfo.getOrderDetails()) {
                    List<CustomerCodeInstall> customerCodeInstallsDetail = new ArrayList<>();

                    if (!StringUtils.isEmpty(orderDetail.getInsideCode2())) {
                        CustomerCodeInstall customerCodeInstall = new CustomerCodeInstall();
                        customerCodeInstall.setBarCode(orderDetail.getInsideCode2());
                        customerCodeInstall.setType("INSIDE");
                        customerCodeInstall.setSpecification(orderDetail.getPname());
                        customerCodeInstallsDetail.add(customerCodeInstall);
                    }
                    if (!StringUtils.isEmpty(orderDetail.getInsideCode())) {
                        CustomerCodeInstall customerCodeInstall = new CustomerCodeInstall();
                        customerCodeInstall.setBarCode(orderDetail.getInsideCode());
                        customerCodeInstall.setType("INSIDE");
                        customerCodeInstall.setSpecification(orderDetail.getPname());
                        customerCodeInstallsDetail.add(customerCodeInstall);
                    }

                    if (!StringUtils.isEmpty(orderDetail.getOutCode())) {
                        CustomerCodeInstall customerCodeInstallOut = new CustomerCodeInstall();
                        customerCodeInstallOut.setBarCode(orderDetail.getOutCode());
                        customerCodeInstallOut.setType("OUT");
                        customerCodeInstallOut.setSpecification(orderDetail.getPname());
                        customerCodeInstallsDetail.add(customerCodeInstallOut);
                    }

                    if (CollectionUtils.isEmpty(customerCodeInstallsDetail))
                        continue;


                    for (CustomerCodeInstall customerCodeInstallDetail : customerCodeInstallsDetail) {
                        List<CustomerOrderItem> collect = customerOrderItems.stream().filter(item -> item.getSpecification().equals(customerCodeInstallDetail.getSpecification())).collect(Collectors.toList());

                        CustomerCodeInstall customerCodeInstall = customerCodeInstallService.lambdaQuery().eq(CustomerCodeInstall::getBarCode, customerCodeInstallDetail.getBarCode()).last("limit 1").one();

                        CustomerCode customerCode = customerCodeService.lambdaQuery().eq(CustomerCode::getBarCode, customerCodeInstallDetail.getBarCode()).last("limit 1").one();
                        customerCodeInstallDetail.setSalesCompany(customerOrder.getCustomerName());
                        if (!CollectionUtils.isEmpty(collect)) {
                            customerCodeInstallDetail.setMaterialCode(collect.get(0).getMaterialCode());


                            if (customerCode != null) {
                                customerCodeInstallDetail.setConsignerTime(customerCode.getConsignerTime());
                                customerCodeInstallDetail.setDeliveryTime(customerCode.getDeliveryTime());
                                customerCodeInstallDetail.setOrderNumber(customerCode.getOrderNumber());
                                customerCodeInstallDetail.setReceiptCompany(customerCode.getDeliveryTime());
                                customerCodeInstallDetail.setAddress(customerCode.getAddress());
                                customerCodeInstallDetail.setWarehouse(customerCode.getWarehouse());
                                customerCodeInstallDetail.setSalesCompany(customerCode.getSalesCompany());
                                customerCodeInstallDetail.setSalesCompanyCode(customerCode.getSalesCompanyCode());
                                customerCodeInstallDetail.setVerificationStatus(1);
                                customerCodeInstallDetail.setConsignerTimeV2(customerCode.getConsignerTimeV2());
                                customerCodeInstallDetail.setDeliveryTimeV2(customerCode.getDeliveryTimeV2());
                            }else {
                                customerCodeInstallDetail.setVerificationStatus(2);


                            }
                            customerCodeInstallDetail.setSalesCompany(customerOrder.getCustomerName());
                            customerCodeInstallDetail.setId(customerCodeInstall != null ? customerCodeInstall.getId():null);
                            customerCodeInstallDetail.setGongDanId(customerOrderInstall.getGongDanId());
                            customerCodeInstallDetail.setSalesNumber(customerOrderInstall.getSalesNumber());
                            customerCodeInstallDetail.setSalesWname(customerOrderInstall.getSalesWname());
                            customerCodeInstallDetail.setInstallName(customerOrderInstall.getInstallName());
                            customerCodeInstallDetail.setInstallNumber(customerOrderInstall.getInstallNumber());
                            customerCodeInstallDetail.setPgid(customerOrderInstall.getPgid());
                            customerCodeInstallDetail.setOrderState(customerOrderInstall.getOrderState());
                            customerCodeInstallDetail.setAppointState(customerOrderInstall.getAppointState());
                            customerCodeInstallDetail.setWorkerName(customerOrderInstall.getWorkerName());
                            customerCodeInstallDetail.setWorkerNumber(customerOrderInstall.getWorkerNumber());
                            customerCodeInstallDetail.setWorkerPhone(customerOrderInstall.getWorkerPhone());
                            customerCodeInstallDetail.setAdminOfficerName(customerOrderInstall.getAdminOfficerName());
                            customerCodeInstallDetail.setAdminOfficerNumber(customerOrderInstall.getAdminOfficerNumber());
                            customerCodeInstallDetail.setCreateNumber(customerOrderInstall.getCreateNumber());
                            customerCodeInstallDetail.setCreateWname(customerOrderInstall.getCreateWname());
                            customerCodeInstallDetail.setAfterNumber(customerOrderInstall.getAfterNumber());
                            customerCodeInstallDetail.setAfterWname(customerOrderInstall.getAfterWname());
                            customerCodeInstallDetail.setAfterTime(customerOrderInstall.getAfterTime());

                            if (customerCodeInstallDetail.getType().equals("INSIDE"))
                                customerCodeInstallDetail.setMaterialType("内机");
                            else
                                customerCodeInstallDetail.setMaterialType("外机");

                        }
                    }

                    customerCodeInstalls.addAll(customerCodeInstallsDetail);

                }
            }

            customerOrderService.saveOrUpdateBatch(customerOrders);
            customerOrderInstallService.saveOrUpdateBatch(customerOrderInstalls);
            customerOrderInstallLogService.saveOrUpdateBatch(customerOrderInstallLogs);
            customerCodeInstallService.saveOrUpdateBatch(customerCodeInstalls);


        }



    }
}
