package com.zfire.mall.manager.schedule;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.zfire.mall.manager.bean.k3.RequestApiBean;
import com.zfire.mall.manager.constant.Constant;
import com.zfire.mall.manager.constant.KingDeeCons;
import com.zfire.mall.manager.exception.RemoteServiceException;
import com.zfire.mall.manager.logic.k3.K3Logic;
import com.zfire.mall.manager.logic.k3.K3PurchaseOrderLogic;
import com.zfire.mall.manager.logic.k3.K3PurchaseStockInLogic;
import com.zfire.mall.manager.logic.k3.K3TransferLogic;
import com.zfire.mall.manager.plus.entity.*;
import com.zfire.mall.manager.plus.service.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.integration.redis.util.RedisLockRegistry;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;


/**
 * 更新库存
 *
 * @author change
 */
@Component
@Slf4j
public class StockInSchedule {

    //预留单
    @Value("${supply.automatic.reserved}")
    private String automaticReservedUrl;

    @Autowired
    StockCustomerService stockCustomerService;
    @Autowired
    StockService stockService;
    @Autowired
    KingDeeOrganizationService kingDeeOrganizationService;
    @Autowired
    K3Logic k3Logic;

    @Autowired
    ReceiptInvoiceAddedService receiptInvoiceAddedService;

    @Autowired
    ReceiptInvoiceOrdinaryService receiptInvoiceOrdinaryService;

    @Autowired
    SalesOrderService salesOrderService;

    @Autowired
    KingDeePurchaseStockInItemService kingDeePurchaseStockInItemService;


    @Autowired
    K3PurchaseOrderLogic k3PurchaseOrderLogic;
    @Autowired
    K3PurchaseStockInLogic k3PurchaseStockInLogic;

    @Autowired
    RedisLockRegistry redisLockRegistry;

    @Autowired
    KingDeePurchaseStockInService kingDeePurchaseStockInService;

    @Autowired
    KingDeePurchaseStockOutService kingDeePurchaseStockOutService;

    @Autowired
    KingDeePurchaseStockOutItemService kingDeePurchaseStockOutItemService;

    @Autowired
    KingDeeOtherStockInService kingDeeOtherStockInService;

    @Autowired
    KingDeeOtherStockInItemService kingDeeOtherStockInItemService;

    @Autowired
    KingDeeOtherStockOutService kingDeeOtherStockOutService;

    @Autowired
    KingDeeOtherStockOutItemService kingDeeOtherStockOutItemService;

    @Autowired
    K3TransferLogic k3TransferLogic;

    @Autowired
    KingDeeTransferService kingDeeTransferService;

    @Autowired
    KingDeeTransferItemService kingDeeTransferItemService;



    final static String orgId = "1";
    final static String orgNum = "100";

    /**
     * 更新实时库存
     */
/*    @Scheduled(fixedDelay = 8 * 1000)
    public void syncInStock() throws Exception {
        // 调用即时库存
        RequestApiBean request = k3Logic.getInstance(null, null, KingDeeCons.STOCK_AMOUNT, null, null);
        String filterStr = "";
        request.setFieldKeys("FID,FStockId,FMaterialId,FBaseQty,FStockOrgId");
        request.setStartTime(null);
        request.setEndTime(null);
        request.setFilterString("");
        request.setOrderString("");

        String[] fields = new String[]{"id", "stockId", "materialId", "stockQty", "orgId"};
        List<Stock> objects = k3Logic.allSync(request, fields, Stock.class);
        if (CollectionUtil.isNotEmpty(objects)) {
            stockService.saveOrUpdateBatch(objects);

        }else {
            throw new RemoteServiceException("数据为空，异常");
        }

    }*/


    /**
     * 超过多少天自动生成预留单库存充足
     */
    @Scheduled(fixedDelay = 60 * 1000 * 60 * 8)
    public void automaticReserved() throws Exception {
        // 更新超过时间的订单生成预留单,物料库存充足
        HttpUtil.get(automaticReservedUrl);
    }

    /**
     * 更新发票
     *
     */
    @Scheduled(fixedDelay = 300 * 1000)
    public void syncRe() throws Exception {
        Date date = new Date();
        Date offset = DateUtil.offset(date, DateField.MONTH, -1);
        //开始时间
        String format = DateUtil.format(offset, "yyyy-MM-dd HH:mm:ss");
        //结束时间
        String formatEnd = DateUtil.format(date, "yyyy-MM-dd HH:mm:ss");

        // 销售增值发票
        RequestApiBean request = k3Logic.getInstance(orgId, orgNum, KingDeeCons.IV_SALESIC, format, formatEnd);
        String filterStr = "";
        request.setFieldKeys("FID,FBillNo,FIVNUMBER,F_kls_Remarks,FINVOICEDATE");
        request.setFilterString(String.format(filterStr, orgNum));
        request.setOrderString("");

        String[] fields = new String[]{"id", "billNo", "fivNumber","comment","invoiceDate"};
        List<ReceiptInvoiceAdded> objects = k3Logic.allSync(request, fields, ReceiptInvoiceAdded.class);


        // 销售普通发票
        RequestApiBean requestOne = k3Logic.getInstance(orgId, orgNum, KingDeeCons.IV_SALESOC, format, formatEnd);
        String filterStrOne = "";
        requestOne.setFieldKeys("FID,FBillNo,FIVNUMBER,F_kls_Remarks,FINVOICEDATE");
        requestOne.setFilterString(String.format(filterStrOne, orgNum));
        requestOne.setOrderString("");

        String[] fieldsOne = new String[]{"id", "billNo", "fivNumber","comment","invoiceDate"};
        List<ReceiptInvoiceOrdinary> objectsOne = k3Logic.allSync(requestOne, fieldsOne, ReceiptInvoiceOrdinary.class);


        if (CollectionUtil.isNotEmpty(objects)) {
            receiptInvoiceAddedService.saveOrUpdateBatch(objects);
        }

        if (CollectionUtil.isNotEmpty(objectsOne)) {
            receiptInvoiceOrdinaryService.saveOrUpdateBatch(objectsOne);
        }

        if (CollectionUtils.isNotEmpty(objects)) {
            for (ReceiptInvoiceAdded object : objects) {
                if (StringUtils.isNotEmpty(object.getComment()) && StrUtil.isNotBlank(object.getFivNumber())) {
                    String[] split = object.getComment().split(",");
                    for (String orderId : split) {
                        SalesOrder salesOrder = salesOrderService.getById(orderId);
                        if (salesOrder == null) {
                            continue;
                        }
                        if (StringUtils.isNotEmpty(salesOrder.getBillReceipt()) && !salesOrder.getBillReceipt().contains(object.getFivNumber())) {
                            salesOrder.setBillReceipt(salesOrder.getBillReceipt() + "/" + object.getFivNumber());
                            salesOrder.setBillDate(object.getInvoiceDate());
                        }

                        if (StringUtils.isEmpty(salesOrder.getBillReceipt())) {
                            salesOrder.setBillReceipt(object.getFivNumber());
                            salesOrder.setBillDate(object.getInvoiceDate());
                        }

                        salesOrder.updateById();
                    }
                }
            }
        }

        if (CollectionUtils.isNotEmpty(objectsOne)) {
            for (ReceiptInvoiceOrdinary object : objectsOne) {
                if (StringUtils.isNotEmpty(object.getComment()) && StrUtil.isNotBlank(object.getFivNumber())) {
                    String[] split = object.getComment().split(",");
                    for (String orderId : split) {
                        SalesOrder salesOrder = salesOrderService.getById(orderId);
                        if (salesOrder == null) {
                            continue;
                        }
                        if (StringUtils.isNotEmpty(salesOrder.getBillReceipt()) && !salesOrder.getBillReceipt().contains(object.getFivNumber())) {
                            salesOrder.setBillReceipt(salesOrder.getBillReceipt() + "/" + object.getFivNumber());
                            salesOrder.setBillDate(object.getInvoiceDate());
                        }

                        if (StringUtils.isEmpty(salesOrder.getBillReceipt())) {
                            salesOrder.setBillReceipt(object.getFivNumber());
                            salesOrder.setBillDate(object.getInvoiceDate());
                        }
                        salesOrder.updateById();
                    }
                }
            }
        }


    }


    @Scheduled(fixedDelay = 30 * 1000)
    public void stockInSync() {
        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.INVOICE_APPLICATION);
        try {
            if (!obtain.tryLock(5, TimeUnit.SECONDS)) {
                throw new RemoteServiceException("系统繁忙，请稍后再试");
            }
            //采购入库单
            purchaseStockInQuanLiang();
            purchaseStockIn();
            //采购退料单
            purchaseStockOutQuanLiang();
            purchaseStockOut();

            //其他入库单
            purchaseStockOtherInQuanLiang();
            purchaseStockOtherIn();
            //其他出库单
            otherStockOutQuanLiang();
            otherStockOut();

            //调拨单
            stockOutInQuanLiang();
            stockOutIn();

        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            obtain.unlock();
        }
    }


    @Transactional(rollbackFor = Exception.class)
    public void stockOutInQuanLiang() throws Exception {

        //全量更新
        // 调用退料单接口
        List<KingDeeTransfer> kingDeeTransfers = kingDeeTransferService.lambdaQuery().eq(KingDeeTransfer::getDocumentStatus, "C")
                .between(KingDeeTransfer::getApproveDate,DateUtil.offset(new Date(), DateField.MONTH, -1),new Date())
                .select(KingDeeTransfer::getId).list();
        if (CollectionUtil.isEmpty(kingDeeTransfers))
            return;
        //获得所有通过的单
        String orderId="";
        for (KingDeeTransfer kingDeeTransfer : kingDeeTransfers) {
            if (StringUtils.isEmpty(orderId)) {
                orderId = kingDeeTransfer.getId();
                continue;
            }
            orderId = orderId+","+kingDeeTransfer.getId();
        }
        RequestApiBean request = k3Logic.getInstance(orgId, orgNum, KingDeeCons.STOCK_TRANSFER, "2002-09-19 16:55:01", "2062-09-19 16:55:01");

        //查找被弃审的单
        String filterStr = "FDocumentStatus != 'C'"+ " AND FID IN ("+orderId+")";
        request.setFieldKeys("FID,FBillNo,FStockOrgId.FName,FStockOutOrgId.FName,FTransferBizType,FNote,FApproverId.FName,FCancellerId.FName,FBillTypeID.FName,FDocumentStatus,FDate,FApproveDate,FCancelDate");
        request.setFilterString(String.format(filterStr, orgNum));
        request.setOrderString("FID ASC");
        request.setNotTime(true);
        String[] fields = new String[]{"id","billNo","stockOrgId","stockOutOrgId","transferBizType","note","approverId","cancellerId","billType","documentStatus","fDate","approveDate","cancelDate"};
        List<KingDeeTransfer> objects = k3Logic.allSync(request, fields, KingDeeTransfer.class);
        /**
         *  全量查询表中不为已审核状态的单，退回数量，再删除数据，等下次审核
         * */
        if (CollectionUtil.isEmpty(objects))
            return;
        for (KingDeeTransfer object : objects) {
            //被弃审的返回数量
            List<KingDeeTransferItem> list = kingDeeTransferItemService.lambdaQuery().eq(KingDeeTransferItem::getFId, object.getId()).list();
            for (KingDeeTransferItem kingDeeTransferItem : list) {
                //查出被反审的内容
                Stock stock = stockService.lambdaQuery().eq(Stock::getStockId, kingDeeTransferItem.getSrcStockId())
                        .eq(Stock::getMaterialId, kingDeeTransferItem.getMaterialId()).last("limit 1").one();

                Stock stockDest = stockService.lambdaQuery().eq(Stock::getStockId, kingDeeTransferItem.getDestStockId())
                        .eq(Stock::getMaterialId, kingDeeTransferItem.getMaterialId()).last("limit 1").one();
                //调出库存加记录
                stockAccLog("全量反审的调拨单仓库加",stock,kingDeeTransferItem.getQty(),kingDeeTransferItem.getFId(),true,"调拨单");
                stock.setStockQty(stock.getStockQty().add(kingDeeTransferItem.getQty()));
                stock.updateById();
                //调入库存减记录
                stockAccLog("全量反审的调拨单仓库减",stockDest,kingDeeTransferItem.getQty(),kingDeeTransferItem.getFId(),false,"调拨单");
                stockDest.setStockQty(stockDest.getStockQty().subtract(kingDeeTransferItem.getQty()));
                stockDest.updateById();
                kingDeeTransferItemService.removeById(kingDeeTransferItem.getId());
            }
            kingDeeTransferService.removeById(object.getId());
        }

    }



    @Transactional(rollbackFor = Exception.class)
    public void stockOutIn() throws Exception {
        //1调拨单入库库存
        List<KingDeeTransferItem> kingDeeTransferItems = allOutInSync(orgId, orgNum, null, null);

        if (CollectionUtil.isNotEmpty(kingDeeTransferItems)) {
            for (KingDeeTransferItem kingDeeTransferItem : kingDeeTransferItems) {
                KingDeeTransferItem byId = kingDeeTransferItemService.getById(kingDeeTransferItem.getId());
                // 入库表表中能找到，相减加库存
                if (byId != null){

                    //新改变仓库 调出扣 调入加
                    Stock stock = stockService.lambdaQuery().eq(Stock::getStockId, kingDeeTransferItem.getSrcStockId())  //调出
                            .eq(Stock::getMaterialId, kingDeeTransferItem.getMaterialId()).last("limit 1").one();
                    stockAccLog("反审的调拨单仓库扣",stock,kingDeeTransferItem.getQty(),kingDeeTransferItem.getFId(),false,"调拨单");
                    stock.setStockQty(stock.getStockQty().subtract(kingDeeTransferItem.getQty()));
                    stock.updateById();

                    Stock stockDest = stockService.lambdaQuery().eq(Stock::getStockId, kingDeeTransferItem.getDestStockId())
                            .eq(Stock::getMaterialId, kingDeeTransferItem.getMaterialId()).last("limit 1").one();  //调入
                    stockAccLog("反审的调拨单仓库加",stockDest,kingDeeTransferItem.getQty(),kingDeeTransferItem.getFId(),true,"调拨单");
                    stockDest.setStockQty(stockDest.getStockQty().add(kingDeeTransferItem.getQty()));
                    stockDest.updateById();

                    //旧改变仓库  调出加  调入扣
                    Stock stockOut = stockService.lambdaQuery().eq(Stock::getStockId, byId.getSrcStockId())
                            .eq(Stock::getMaterialId, kingDeeTransferItem.getMaterialId()).last("limit 1").one();  //调出
                    stockAccLog("反审的调拨单仓库加",stockOut,kingDeeTransferItem.getQty(),kingDeeTransferItem.getFId(),true,"调拨单");
                    stockOut.setStockQty(stockOut.getStockQty().add(kingDeeTransferItem.getQty()));
                    stockOut.updateById();


                    Stock stockIn = stockService.lambdaQuery().eq(Stock::getStockId, byId.getDestStockId())
                            .eq(Stock::getMaterialId, kingDeeTransferItem.getMaterialId()).last("limit 1").one();  //调入
                    stockAccLog("反审的调拨单仓库扣",stockIn,kingDeeTransferItem.getQty(),kingDeeTransferItem.getFId(),false,"调拨单");
                    stockIn.setStockQty(stockIn.getStockQty().subtract(kingDeeTransferItem.getQty()));
                    stockIn.updateById();

                }


                // 入库表不能找到，查看库存表有无记录 ，无库存记录找库存，有库存记录相加
                if (byId == null){
                    //新仓库加减
                    Stock stock = stockService.lambdaQuery().eq(Stock::getStockId, kingDeeTransferItem.getSrcStockId())
                            .eq(Stock::getMaterialId, kingDeeTransferItem.getMaterialId()).last("limit 1").one();

                    Stock stockDest = stockService.lambdaQuery().eq(Stock::getStockId, kingDeeTransferItem.getDestStockId())
                            .eq(Stock::getMaterialId, kingDeeTransferItem.getMaterialId()).last("limit 1").one();
                    //仓库有记录
                    if (stockDest != null){
                        //库存加减记录
                        stockAccLog("调拨单仓库扣",stock,kingDeeTransferItem.getQty(),kingDeeTransferItem.getFId(),false,"调拨单");
                        //调出扣库存
                        stock.setStockQty(stock.getStockQty().subtract(kingDeeTransferItem.getQty()));
                        stock.updateById();

                        //调入仓库加库存
                        stockAccLog("调拨单仓库加",stock,kingDeeTransferItem.getQty(),kingDeeTransferItem.getFId(),true,"调拨单");
                        stockDest.setStockQty(stockDest.getStockQty().add(kingDeeTransferItem.getQty()));
                        stockDest.updateById();

                    }
                    //仓库库存记录无记录
                    if (stockDest == null){

                        //库存加减记录
                        stockAccLog("调拨单仓库扣",stock,kingDeeTransferItem.getQty(),kingDeeTransferItem.getFId(),false,"调拨单");
                        //调出扣库存
                        stock.setStockQty(stock.getStockQty().subtract(kingDeeTransferItem.getQty()));
                        stock.updateById();


                        Stock stockIn = stockId( kingDeeTransferItem.getDestStockId(),kingDeeTransferItem.getMaterialId());
                        stockIn.setStockQty(new BigDecimal("0"));
                        //调入仓库加库存
                        stockAccLog("调拨单仓库加",stockIn,kingDeeTransferItem.getQty(),kingDeeTransferItem.getFId(),true,"调拨单");
                        stockIn.setStockQty(stockIn.getStockQty().add(kingDeeTransferItem.getQty()));
                        stockIn.insert();
                    }
                }
            }

            kingDeeTransferItemService.saveOrUpdateBatch(kingDeeTransferItems);
        }

    }


    public List<KingDeeTransferItem> allOutInSync(String orgId, String orgNumber, String startTime, String endTime) throws Exception {
        // 调用入库单接口

        RequestApiBean request = k3Logic.getInstance(orgId, orgNumber, KingDeeCons.STOCK_TRANSFER, startTime, endTime);
        String filterStr = "FDocumentStatus = 'C' "+ k3Logic.supplySql("FApproveDate", request.getStartTime(), request.getEndTime());
        request.setFieldKeys("FID,FBillNo,FStockOrgId.FName,FStockOutOrgId.FName,FTransferBizType,FNote,FApproverId.FName,FCancellerId.FName,FBillTypeID.FName,FDocumentStatus,FDate,FApproveDate,FCancelDate");
        request.setFilterString(String.format(filterStr, orgNum));
        request.setOrderString("FID ASC");
        request.setNotTime(true);
        String[] fields = new String[]{"id","billNo","stockOrgId","stockOutOrgId","transferBizType","note","approverId","cancellerId","billType","documentStatus","fDate","approveDate","cancelDate"};
        List<KingDeeTransfer> objects = k3Logic.allSync(request, fields, KingDeeTransfer.class);

        if (!org.springframework.util.CollectionUtils.isEmpty(objects)) {
            kingDeeTransferService.saveOrUpdateBatch(objects);
        }

        return allOutInSyncOrder(orgId,orgNumber,startTime,endTime);
    }


    public List<KingDeeTransferItem> allOutInSyncOrder(String orgId, String orgNumber, String startTime, String endTime) throws Exception {
        // 调用入库单接口
        RequestApiBean request = k3Logic.getInstance(orgId, orgNumber, KingDeeCons.STOCK_TRANSFER, startTime, endTime);
        String filterStr = "FDocumentStatus = 'C' "+ k3Logic.supplySql("FApproveDate", request.getStartTime(), request.getEndTime());
        request.setFieldKeys("FBillEntry_FEntryID,FID,FBillNo,FMaterialId,FMaterialId.FNumber,FMaterialName,FSrcStockId,FDestStockId,FQty,FOwnerId,FOwnerOutId," +
                "FModel,FBaseUnitId.FName,FConsignAmount,FConsignPrice,FSrcStockLocId,FDestStockLocId,FAuxPropId,FSrcStockId.FName,FDestStockId.FName,FNoteEntry");
        request.setFilterString(String.format(filterStr, orgNumber));
        request.setOrderString("FID ASC");

        String[] fields = new String[]{"id","fId","billNo","materialId","materialNumber","materialName","srcStockId","destStockId","qty","ownerId","ownerOutId",
                "model","baseUnitId","consignAmount","consignPrice","srcStockLocId","destStockLocId","auxPropId","srcStockName","destStockName","noteEntry"};
        List<KingDeeTransferItem> objects = k3Logic.allSync(request, fields, KingDeeTransferItem.class);
/*        if (CollectionUtil.isNotEmpty(objects)) {
            kingDeePurchaseStockInItemService.saveOrUpdateBatch(objects);
        }*/
        return objects;
    }




    @Transactional(rollbackFor = Exception.class)
    public void otherStockOutQuanLiang() throws Exception {
        //全量更新
        // 调用退料单接口
        List<KingDeeOtherStockOut> kingDeeOtherStockOuts = kingDeeOtherStockOutService.lambdaQuery().eq(KingDeeOtherStockOut::getFdDocumentStatus, "C")
                .between(KingDeeOtherStockOut::getApproveDate,DateUtil.offset(new Date(), DateField.MONTH, -1),new Date())
                .select(KingDeeOtherStockOut::getId).list();
        if (CollectionUtil.isEmpty(kingDeeOtherStockOuts))
            return;
        //获得所有通过的单
        String orderId="";
        for (KingDeeOtherStockOut kingDeePurchaseStockOut : kingDeeOtherStockOuts) {
            if (StringUtils.isEmpty(orderId)) {
                orderId = kingDeePurchaseStockOut.getId();
                continue;
            }
            orderId = orderId+","+kingDeePurchaseStockOut.getId();
        }
        RequestApiBean request = k3Logic.getInstance(orgId, orgNum, KingDeeCons.OTHER_OUT_STOCK, "2002-09-19 16:55:01", "2062-09-19 16:55:01");

        //查找被弃审的单
        String filterStr = "FDocumentStatus != 'C'"+ " AND FID IN ("+orderId+")";
        request.setFieldKeys("FID,FBillNo,FDate,FStockOrgId,FApproverId.FName,FApproveDate,FCancelDate," +
                "FNote,FDocumentStatus,FStockDirect");
        request.setFilterString(String.format(filterStr, orgNum));
        request.setOrderString("FID ASC");

        String[] fields = new String[]{"id","billNo","fDate","stockOrgId","approverId","approveDate","cancelDate",
                "note","fdDocumentStatus","stockDirect"};
        request.setNotTime(true);
        List<KingDeeOtherStockOut> objects = k3Logic.allSync(request, fields, KingDeeOtherStockOut.class);
        /**
         *  全量查询表中不为已审核状态的单，退回数量，再删除数据，等下次审核
         * */
        if (CollectionUtil.isEmpty(objects))
            return;
        for (KingDeeOtherStockOut object : objects) {
            //被弃审的返回数量
            List<KingDeeOtherStockOutItem> list = kingDeeOtherStockOutItemService.lambdaQuery().eq(KingDeeOtherStockOutItem::getFId, object.getId()).list();
            for (KingDeeOtherStockOutItem kingDeeOtherStockOutItem : list) {
                //查出被反审的内容
                Stock stock = stockService.lambdaQuery().eq(Stock::getStockId, kingDeeOtherStockOutItem.getStockId())
                        .eq(Stock::getMaterialId, kingDeeOtherStockOutItem.getMaterialId()).last("limit 1").one();
                if (object.getStockDirect().equals(Constant.OTHER_STOCK_IN)) {
                    //库存减记录
                    stockAccLog("全量反审的其他出库单仓库加", stock, kingDeeOtherStockOutItem.getQty(), kingDeeOtherStockOutItem.getFId(), true, "其他出库单");
                    stock.setStockQty(stock.getStockQty().add(kingDeeOtherStockOutItem.getQty()));
                    stock.updateById();
                    kingDeeOtherStockOutItemService.removeById(kingDeeOtherStockOutItem.getId());
                }
                if (object.getStockDirect().equals(Constant.OTHER_STOCK_OUT)){
                    //库存减记录
                    stockAccLog("全量反审的其他出库单退货仓库加", stock, kingDeeOtherStockOutItem.getQty(), kingDeeOtherStockOutItem.getFId(), false, "其他出库单");
                    stock.setStockQty(stock.getStockQty().subtract(kingDeeOtherStockOutItem.getQty()));
                    stock.updateById();
                    kingDeeOtherStockOutItemService.removeById(kingDeeOtherStockOutItem.getId());
                }
            }
            kingDeeOtherStockOutService.removeById(object.getId());
        }

    }



    @Transactional(rollbackFor = Exception.class)
    public void otherStockOut() throws Exception {
        //1采购退料单入库库存
        List<KingDeeOtherStockOutItem> kingDeePurchaseStockOuts = allOtherOutSync(orgId, orgNum, null, null);

        if (CollectionUtil.isNotEmpty(kingDeePurchaseStockOuts)) {
            for (KingDeeOtherStockOutItem kingDeePurchaseStockOutItem : kingDeePurchaseStockOuts) {
                KingDeeOtherStockOutItem byId = kingDeeOtherStockOutItemService.getById(kingDeePurchaseStockOutItem.getId());
                KingDeeOtherStockOut kingDeeOtherStockOut = kingDeeOtherStockOutService.getById(kingDeePurchaseStockOutItem.getFId());
                // 入库表表中能找到，相减加库存
                if (byId != null){

                    Stock stock = stockService.lambdaQuery().eq(Stock::getStockId, byId.getStockId())
                            .eq(Stock::getMaterialId, byId.getMaterialId()).last("limit 1").one();

                    //仓库相同  相减加减库存
                    if (byId.getStockId().equals(kingDeePurchaseStockOutItem.getStockId())){

                        //加库存数量
                        BigDecimal subtract = byId.getQty().subtract(kingDeePurchaseStockOutItem.getQty());

                        //仓库库存记录有记录
                        if (stock != null){
                            if (kingDeeOtherStockOut.getStockDirect().equals(Constant.OTHER_STOCK_IN)) {
                                //库存加减记录
                                stockAccLog("反审的其他出库单相同仓库加减", stock, subtract.compareTo(BigDecimal.ZERO) < 0 ? (subtract.multiply(new BigDecimal("-1"))) : subtract, kingDeePurchaseStockOutItem.getFId(), subtract.compareTo(BigDecimal.ZERO) < 0 ? false : true, "其他出库单");
                                //理解     退料反审   前一个为2  反审再审 3（相当于比反审前多退一个）   库存反审前是5    5+（2-3）
                                stock.setStockQty(stock.getStockQty().add(subtract));
                                stock.updateById();
                            }
                            if (kingDeeOtherStockOut.getStockDirect().equals(Constant.OTHER_STOCK_OUT)){
                                //库存加减记录
                                stockAccLog("反审的其他出库单退货相同仓库加减", stock, subtract.compareTo(BigDecimal.ZERO) < 0 ? (subtract.multiply(new BigDecimal("-1"))) : subtract, kingDeePurchaseStockOutItem.getFId(), subtract.compareTo(BigDecimal.ZERO) < 0 ? true : false, "其他出库单");
                                //理解     退料反审   前一个为2  反审再审 3（相当于比反审前多退一个）   库存反审前是5    5+（2-3）
                                stock.setStockQty(stock.getStockQty().subtract(subtract));
                                stock.updateById();
                            }
                        }
                        //仓库库存记录无记录  不会发生
                        if (stock == null){

                        }
                    }

                    //仓库不同 先扣回之前仓库，再加现库存
                    if (!byId.getStockId().equals(kingDeePurchaseStockOutItem.getStockId())){

                        //减除之前仓库 退料加回数量
                        if (stock != null){
                            if (kingDeeOtherStockOut.getStockDirect().equals(Constant.OTHER_STOCK_IN)) {
                                //库存加减记录
                                stockAccLog("反审的其他出库单不同仓库加", stock, byId.getQty(), kingDeePurchaseStockOutItem.getFId(), true, "其他出库单");
                                //减少回之前的库存
                                stock.setStockQty(stock.getStockQty().add(byId.getQty()));
                                stock.updateById();
                            }
                            if (kingDeeOtherStockOut.getStockDirect().equals(Constant.OTHER_STOCK_OUT)){
                                //库存加减记录
                                stockAccLog("反审的其他出库单退货不同仓库减少", stock, byId.getQty(), kingDeePurchaseStockOutItem.getFId(), false, "其他出库单");
                                //减少回之前的库存
                                stock.setStockQty(stock.getStockQty().subtract(byId.getQty()));
                                stock.updateById();
                            }

                        }

                        //新仓库加减
                        Stock stockNew = stockService.lambdaQuery().eq(Stock::getStockId, kingDeePurchaseStockOutItem.getStockId())
                                .eq(Stock::getMaterialId, kingDeePurchaseStockOutItem.getMaterialId()).last("limit 1").one();
                        //仓库有记录
                        if (stockNew != null){
                            if (kingDeeOtherStockOut.getStockDirect().equals(Constant.OTHER_STOCK_IN)) {
                                //库存加减记录
                                stockAccLog("反审的其他出库单不同仓库减少", stockNew, kingDeePurchaseStockOutItem.getQty(), kingDeePurchaseStockOutItem.getFId(), false, "其他出库单");
                                //减少库存
                                stockNew.setStockQty(stockNew.getStockQty().subtract(kingDeePurchaseStockOutItem.getQty()));
                                stockNew.updateById();
                            }
                            if (kingDeeOtherStockOut.getStockDirect().equals(Constant.OTHER_STOCK_OUT)){
                                //库存加减记录
                                stockAccLog("反审的其他出库单退货不同仓库加", stockNew, kingDeePurchaseStockOutItem.getQty(), kingDeePurchaseStockOutItem.getFId(), true, "其他出库单");
                                //减少库存
                                stockNew.setStockQty(stockNew.getStockQty().add(kingDeePurchaseStockOutItem.getQty()));
                                stockNew.updateById();
                            }

                        }
                        //仓库库存记录无记录 不存在这种情况
                        if (stockNew == null){

                        }
                    }

                }


                // 入库表不能找到，查看库存表有无记录 ，无库存记录找库存，有库存记录相加
                if (byId == null){
                    //新仓库加减
                    Stock stockNew = stockService.lambdaQuery().eq(Stock::getStockId, kingDeePurchaseStockOutItem.getStockId())
                            .eq(Stock::getMaterialId, kingDeePurchaseStockOutItem.getMaterialId()).last("limit 1").one();
                    //仓库有记录
                    if (stockNew != null){
                        if (kingDeeOtherStockOut.getStockDirect().equals(Constant.OTHER_STOCK_IN)) {
                            //库存加减记录
                            stockAccLog("其他出库单仓库减", stockNew, kingDeePurchaseStockOutItem.getQty(), kingDeePurchaseStockOutItem.getFId(), false, "其他出库单");
                            //加库存
                            stockNew.setStockQty(stockNew.getStockQty().subtract(kingDeePurchaseStockOutItem.getQty()));
                            stockNew.updateById();
                        }
                        if (kingDeeOtherStockOut.getStockDirect().equals(Constant.OTHER_STOCK_OUT)) {
                            //库存加减记录
                            stockAccLog("其他出库单仓库加", stockNew, kingDeePurchaseStockOutItem.getQty(), kingDeePurchaseStockOutItem.getFId(), true, "其他出库单");
                            //加库存
                            stockNew.setStockQty(stockNew.getStockQty().add(kingDeePurchaseStockOutItem.getQty()));
                            stockNew.updateById();
                        }

                        }
                    //仓库库存记录无记录 不存在这种可能
                    if (stockNew == null){

                    }
                }
            }

            kingDeeOtherStockOutItemService.saveOrUpdateBatch(kingDeePurchaseStockOuts);
        }

    }


    public List<KingDeeOtherStockOutItem> allOtherOutSync(String orgId, String orgNumber, String startTime, String endTime) throws Exception {
        // 调用入库单接口
        RequestApiBean request = k3Logic.getInstance(orgId, orgNumber, KingDeeCons.OTHER_OUT_STOCK, startTime, endTime);
        String filterStr = "FDocumentStatus = 'C' "+ k3Logic.supplySql("FApproveDate", request.getStartTime(), request.getEndTime());
        request.setFieldKeys("FID,FBillNo,FDate,FStockOrgId,FApproverId.FName,FApproveDate,FCancelDate," +
                "FNote,FDocumentStatus,FStockDirect");
        request.setFilterString(String.format(filterStr, orgNum));
        request.setOrderString("FID ASC");

        String[] fields = new String[]{"id","billNo","fDate","stockOrgId","approverId","approveDate","cancelDate",
                "note","fdDocumentStatus","stockDirect"};
        request.setNotTime(true);
        List<KingDeeOtherStockOut> objects = k3Logic.allSync(request, fields, KingDeeOtherStockOut.class);
        if (!org.springframework.util.CollectionUtils.isEmpty(objects)) {
            kingDeeOtherStockOutService.saveOrUpdateBatch(objects);
        }

        return allSyncOtherOutOrder(orgId,orgNumber,startTime,endTime);
    }


    public List<KingDeeOtherStockOutItem> allSyncOtherOutOrder(String orgId, String orgNumber, String startTime, String endTime) throws Exception {
        // 调用入库单接口
        RequestApiBean request = k3Logic.getInstance(orgId, orgNumber, KingDeeCons.OTHER_OUT_STOCK, startTime, endTime);
        String filterStr = "FDocumentStatus = 'C' "+ k3Logic.supplySql("FApproveDate", request.getStartTime(), request.getEndTime());
        request.setFieldKeys("FEntity_FEntryID,FID,FBillNo,FMaterialId,FMaterialId.FNumber,FMaterialId.FName," +
                "FStockId,FEntryNote,FQty");
        request.setFilterString(String.format(filterStr, orgNumber));
        request.setOrderString("FID ASC");

        String[] fields = new String[]{"id","fId","billNo","materialId","materialNumber","materialName","stockId","entryNote","qty"};
        List<KingDeeOtherStockOutItem> objects = k3Logic.allSync(request, fields, KingDeeOtherStockOutItem.class);
/*        if (CollectionUtil.isNotEmpty(objects)) {
            kingDeePurchaseStockInItemService.saveOrUpdateBatch(objects);
        }*/
        return objects;
    }



    @Transactional(rollbackFor = Exception.class)
    public void purchaseStockOtherInQuanLiang() throws Exception {
        //全量更新
        // 调用入库单接口
        List<KingDeeOtherStockIn> kingDeeOtherStockIns = kingDeeOtherStockInService.lambdaQuery().eq(KingDeeOtherStockIn::getFdDocumentStatus, "C")
                .between(KingDeeOtherStockIn::getApproveDate,DateUtil.offset(new Date(), DateField.MONTH, -1),new Date())
                .select(KingDeeOtherStockIn::getId).list();
        if (CollectionUtil.isEmpty(kingDeeOtherStockIns))
            return;
        //获得所有通过的单
        String orderId="";
        for (KingDeeOtherStockIn kingDeeOtherStockIn : kingDeeOtherStockIns) {
            if (StringUtils.isEmpty(orderId)) {
                orderId = kingDeeOtherStockIn.getId();
                continue;
            }
            orderId = orderId+","+kingDeeOtherStockIn.getId();
        }
        RequestApiBean request = k3Logic.getInstance(orgId, orgNum, KingDeeCons.OTHER_IN_STOCK, "2002-09-19 16:55:01", "2062-09-19 16:55:01");

        //查找被弃审的单
        String filterStr = "FDocumentStatus != 'C'"+ " AND FID IN ("+orderId+")";
        request.setFieldKeys("FID,FBillNo,FDate,FStockOrgId,FSupplierId,FApproverId.FName,FApproveDate,FCancellerId,FCancelDate," +
                "FNOTE,FDocumentStatus,FStockDirect");
        request.setFilterString(String.format(filterStr, orgNum));
        request.setOrderString("FID ASC");

        String[] fields = new String[]{"id","billNo","fDate","stockOrgId","supplierId","approverId","approveDate","cancellerId","cancelDate",
                "note","fdDocumentStatus","stockDirect"};
        request.setNotTime(true);
        List<KingDeeOtherStockIn> objects = k3Logic.allSync(request, fields, KingDeeOtherStockIn.class);
        /**
         *  全量查询表中不为已审核状态的单，退回数量，再删除数据，等下次审核
         * */
        if (CollectionUtil.isEmpty(objects))
            return;
        for (KingDeeOtherStockIn object : objects) {
            //被弃审的返回数量
            List<KingDeeOtherStockInItem> list = kingDeeOtherStockInItemService.lambdaQuery().eq(KingDeeOtherStockInItem::getFId, object.getId()).list();
            for (KingDeeOtherStockInItem kingDeeOtherStockIn : list) {
                //查出被反审的内容
                Stock stock = stockService.lambdaQuery().eq(Stock::getStockId, kingDeeOtherStockIn.getStockId())
                        .eq(Stock::getMaterialId, kingDeeOtherStockIn.getMaterialId()).last("limit 1").one();
                if (object.getStockDirect().equals(Constant.OTHER_STOCK_IN)) {
                    //库存减记录
                    stockAccLog("全量反审的其他入库单仓库减", stock, kingDeeOtherStockIn.getQty(), kingDeeOtherStockIn.getFId(), false, "其他入库单");
                    stock.setStockQty(stock.getStockQty().subtract(kingDeeOtherStockIn.getQty()));
                    stock.updateById();
                    kingDeeOtherStockInItemService.removeById(kingDeeOtherStockIn.getId());
                }
                if (object.getStockDirect().equals(Constant.OTHER_STOCK_OUT)){
                    //库存加记录
                    stockAccLog("全量反审的其他入库单退货仓库加", stock, kingDeeOtherStockIn.getQty(), kingDeeOtherStockIn.getFId(), true, "其他入库单");
                    stock.setStockQty(stock.getStockQty().add(kingDeeOtherStockIn.getQty()));
                    stock.updateById();
                    kingDeeOtherStockInItemService.removeById(kingDeeOtherStockIn.getId());
                }
            }
            kingDeeOtherStockInService.removeById(object.getId());
        }

    }




    @Transactional(rollbackFor = Exception.class)
    public void purchaseStockOtherIn() throws Exception {
        //1采购入库单入库库存
        List<KingDeeOtherStockInItem> kingDeeOtherStockIns = allSyncOtherIn(orgId, orgNum, null, null);

        if (CollectionUtil.isNotEmpty(kingDeeOtherStockIns)) {
            for (KingDeeOtherStockInItem kingDeeOtherStockInItem : kingDeeOtherStockIns) {
                KingDeeOtherStockInItem byId = kingDeeOtherStockInItemService.getById(kingDeeOtherStockInItem.getId());
                KingDeeOtherStockIn kingDeeOtherStockIn = kingDeeOtherStockInService.getById(kingDeeOtherStockInItem.getFId());

                // 入库表表中能找到，相减加库存
                if (byId != null){

                    Stock stock = stockService.lambdaQuery().eq(Stock::getStockId, byId.getStockId())
                            .eq(Stock::getMaterialId, byId.getMaterialId()).last("limit 1").one();

                    //仓库相同  相减加减库存
                    if (byId.getStockId().equals(kingDeeOtherStockInItem.getStockId())){

                        //加库存数量
                        BigDecimal subtract = kingDeeOtherStockInItem.getQty().subtract(byId.getQty());

                        //仓库库存记录有记录
                        if (stock != null){
                            //库存加减记录
                            if (kingDeeOtherStockIn.getStockDirect().equals(Constant.OTHER_STOCK_IN)) {
                                stockAccLog("反审的其他入库单相同仓库加减", stock, subtract.compareTo(BigDecimal.ZERO) < 0 ? (subtract.multiply(new BigDecimal("-1"))) : subtract, kingDeeOtherStockInItem.getFId(), subtract.compareTo(BigDecimal.ZERO) < 0 ? false : true, "其他入库单");
                                stock.setStockQty(stock.getStockQty().add(subtract));
                                stock.updateById();
                            }
                            if (kingDeeOtherStockIn.getStockDirect().equals(Constant.OTHER_STOCK_OUT)){
                                stockAccLog("反审的其他入库单退货相同仓库加减", stock, subtract.compareTo(BigDecimal.ZERO) < 0 ? (subtract.multiply(new BigDecimal("-1"))) : subtract, kingDeeOtherStockInItem.getFId(), subtract.compareTo(BigDecimal.ZERO) < 0 ? true : false, "其他入库单");
                                stock.setStockQty(stock.getStockQty().subtract(subtract));
                                stock.updateById();
                            }
                        }
                        //仓库库存记录无记录  不会发生
                        if (stock == null){

                        }
                    }

                    //仓库不同 先扣回之前仓库，再加现库存
                    if (!byId.getStockId().equals(kingDeeOtherStockInItem.getStockId())){

                        //减除之前仓库
                        if (stock != null){
                            //库存加减记录
                            if (kingDeeOtherStockIn.getStockDirect().equals(Constant.OTHER_STOCK_IN)) {

                                stockAccLog("反审的其他入库单不同仓库减", stock, byId.getQty(), kingDeeOtherStockInItem.getFId(), false, "其他入库单");
                                //减少回之前的库存
                                stock.setStockQty(stock.getStockQty().subtract(byId.getQty()));
                                stock.updateById();
                            }
                            if (kingDeeOtherStockIn.getStockDirect().equals(Constant.OTHER_STOCK_OUT)) {

                                stockAccLog("反审的其他入库单不同仓库退货加", stock, byId.getQty(), kingDeeOtherStockInItem.getFId(), true, "其他入库单");
                                //减少回之前的库存
                                stock.setStockQty(stock.getStockQty().add(byId.getQty()));
                                stock.updateById();
                            }

                        }

                        //新仓库加减
                        Stock stockNew = stockService.lambdaQuery().eq(Stock::getStockId, kingDeeOtherStockInItem.getStockId())
                                .eq(Stock::getMaterialId, kingDeeOtherStockInItem.getMaterialId()).last("limit 1").one();
                        //仓库有记录
                        if (stockNew != null){
                            if (kingDeeOtherStockIn.getStockDirect().equals(Constant.OTHER_STOCK_IN)) {
                                //库存加减记录
                                stockAccLog("反审的其他入库单不同仓库加", stockNew, kingDeeOtherStockInItem.getQty(), kingDeeOtherStockInItem.getFId(), true, "其他入库单");
                                //加库存
                                stockNew.setStockQty(stockNew.getStockQty().add(kingDeeOtherStockInItem.getQty()));
                                stockNew.updateById();
                            }
                            if (kingDeeOtherStockIn.getStockDirect().equals(Constant.OTHER_STOCK_OUT)){
                                //库存加减记录
                                stockAccLog("反审的其他入库单退货不同仓库减", stockNew, kingDeeOtherStockInItem.getQty(), kingDeeOtherStockInItem.getFId(), false, "其他入库单");
                                //加库存
                                stockNew.setStockQty(stockNew.getStockQty().subtract(kingDeeOtherStockInItem.getQty()));
                                stockNew.updateById();
                            }

                        }
                        //仓库库存记录无记录
                        if (stockNew == null){
                            Stock stockIn = stockId(kingDeeOtherStockInItem.getStockId(),kingDeeOtherStockInItem.getMaterialId());
                            stockIn.setStockQty(new BigDecimal("0"));

                            if (kingDeeOtherStockIn.getStockDirect().equals(Constant.OTHER_STOCK_IN)) {
                                //库存加减记录
                                stockAccLog("反审的其他入库单不同仓库加", stockIn, kingDeeOtherStockInItem.getQty(), kingDeeOtherStockInItem.getFId(), true, "其他入库单");
                                //加库存
                                stockIn.setStockQty(kingDeeOtherStockInItem.getQty());
                                stockIn.insert();
                            }

                        }
                    }

                }


                // 入库表不能找到，查看库存表有无记录 ，无库存记录找库存，有库存记录相加
                if (byId == null){
                    //新仓库加减
                    Stock stockNew = stockService.lambdaQuery().eq(Stock::getStockId, kingDeeOtherStockInItem.getStockId())
                            .eq(Stock::getMaterialId, kingDeeOtherStockInItem.getMaterialId()).last("limit 1").one();
                    //仓库有记录
                    if (stockNew != null){
                        if (kingDeeOtherStockIn.getStockDirect().equals(Constant.OTHER_STOCK_IN)) {

                            //库存加减记录
                            stockAccLog("其他入库单仓库加", stockNew, kingDeeOtherStockInItem.getQty(), kingDeeOtherStockInItem.getFId(), true, "其他入库单");
                            //加库存
                            stockNew.setStockQty(stockNew.getStockQty().add(kingDeeOtherStockInItem.getQty()));
                            stockNew.updateById();
                        }

                        if (kingDeeOtherStockIn.getStockDirect().equals(Constant.OTHER_STOCK_OUT)) {

                            //库存加减记录
                            stockAccLog("其他入库单仓库减少", stockNew, kingDeeOtherStockInItem.getQty(), kingDeeOtherStockInItem.getFId(), true, "其他入库单");
                            //加库存
                            stockNew.setStockQty(stockNew.getStockQty().subtract(kingDeeOtherStockInItem.getQty()));
                            stockNew.updateById();
                        }

                    }
                    //仓库库存记录无记录
                    if (stockNew == null){

                        Stock stockIn = stockId( kingDeeOtherStockInItem.getStockId(),kingDeeOtherStockInItem.getMaterialId());
                        stockIn.setStockQty(new BigDecimal("0"));
                        //库存加减记录
                        stockAccLog("入库单不同仓库加",stockIn,kingDeeOtherStockInItem.getQty(),kingDeeOtherStockInItem.getFId(),true,"其他入库单");
                        //加库存
                        stockIn.setStockQty(kingDeeOtherStockInItem.getQty());
                        stockIn.insert();

                    }
                }
            }

            kingDeeOtherStockInItemService.saveOrUpdateBatch(kingDeeOtherStockIns);
        }

    }



    public List<KingDeeOtherStockInItem> allSyncOtherIn(String orgId, String orgNumber, String startTime, String endTime) throws Exception {
        // 调用入库单接口
        RequestApiBean request = k3Logic.getInstance(orgId, orgNumber, KingDeeCons.OTHER_IN_STOCK, startTime, endTime);
        String filterStr = "FDocumentStatus = 'C' "+ k3Logic.supplySql("FApproveDate", request.getStartTime(), request.getEndTime());
        request.setFieldKeys("FID,FBillNo,FDate,FStockOrgId,FSupplierId,FApproverId.FName,FApproveDate,FCancellerId,FCancelDate," +
                "FNOTE,FDocumentStatus,FStockDirect");
        request.setFilterString(String.format(filterStr, orgNum));
        request.setOrderString("FID ASC");
        request.setNotTime(true);

        String[] fields = new String[]{"id","billNo","fDate","stockOrgId","supplierId","approverId","approveDate","cancellerId","cancelDate",
                "note","fdDocumentStatus","stockDirect"};
        List<KingDeeOtherStockIn> objects = k3Logic.allSync(request, fields, KingDeeOtherStockIn.class);
        if (!org.springframework.util.CollectionUtils.isEmpty(objects)) {
            kingDeeOtherStockInService.saveOrUpdateBatch(objects);
        }

        return allSyncOrderOtherIn(orgId,orgNumber,startTime,endTime);
    }


    public List<KingDeeOtherStockInItem> allSyncOrderOtherIn(String orgId, String orgNumber, String startTime, String endTime) throws Exception {
        // 调用入库单接口
        RequestApiBean request = k3Logic.getInstance(orgId, orgNumber, KingDeeCons.OTHER_IN_STOCK, startTime, endTime);
        String filterStr = "FDocumentStatus = 'C' "+ k3Logic.supplySql("FApproveDate", request.getStartTime(), request.getEndTime());
        request.setFieldKeys("FEntity_FEntryID,FID,FBillNo,FMaterialId,FMaterialId.FNumber,FMaterialId.FName,FStockId,FEntryNote,FQty");
        request.setFilterString(String.format(filterStr, orgNumber));
        request.setOrderString("FID ASC");

        String[] fields = new String[]{"id","fId","billNo","materialId","materialNumber","materialName","stockId","entryNote","qty"};
        List<KingDeeOtherStockInItem> objects = k3Logic.allSync(request, fields, KingDeeOtherStockInItem.class);
/*        if (CollectionUtil.isNotEmpty(objects)) {
            kingDeePurchaseStockInItemService.saveOrUpdateBatch(objects);
        }*/
        return objects;
    }

    @Transactional(rollbackFor = Exception.class)
    public void purchaseStockOut() throws Exception {
        //1采购退料单入库库存
        List<KingDeePurchaseStockOutItem> kingDeePurchaseStockOuts = allOutSync(orgId, orgNum, null, null);

        if (CollectionUtil.isNotEmpty(kingDeePurchaseStockOuts)) {
            for (KingDeePurchaseStockOutItem kingDeePurchaseStockOutItem : kingDeePurchaseStockOuts) {
                KingDeePurchaseStockOutItem byId = kingDeePurchaseStockOutItemService.getById(kingDeePurchaseStockOutItem.getId());
                // 入库表表中能找到，相减加库存
                if (byId != null){

                    Stock stock = stockService.lambdaQuery().eq(Stock::getStockId, byId.getStockId())
                            .eq(Stock::getMaterialId, byId.getMaterialId()).last("limit 1").one();

                    //仓库相同  相减加减库存
                    if (byId.getStockId().equals(kingDeePurchaseStockOutItem.getStockId())){

                        //加库存数量
                        BigDecimal subtract = byId.getRmRealQty().subtract(kingDeePurchaseStockOutItem.getRmRealQty());

                        //仓库库存记录有记录
                        if (stock != null){
                            //库存加减记录
                            stockAccLog("反审的退料单相同仓库加减",stock,subtract.compareTo(BigDecimal.ZERO) < 0 ?(subtract.multiply(new BigDecimal("-1"))):subtract,kingDeePurchaseStockOutItem.getFId(),subtract.compareTo(BigDecimal.ZERO) < 0 ?false:true,"采购退料单");
                            //理解     退料反审   前一个为2  反审再审 3（相当于比反审前多退一个）   库存反审前是5    5+（2-3）
                            stock.setStockQty(stock.getStockQty().add(subtract));
                            stock.updateById();
                        }
                        //仓库库存记录无记录  不会发生
                        if (stock == null){

                        }
                    }

                    //仓库不同 先扣回之前仓库，再加现库存
                    if (!byId.getStockId().equals(kingDeePurchaseStockOutItem.getStockId())){

                        //减除之前仓库 退料加回数量
                        if (stock != null){
                            //库存加减记录
                            stockAccLog("反审的退料单不同仓库加",stock,byId.getRmRealQty(),kingDeePurchaseStockOutItem.getFId(),true,"采购退料单");
                            //减少回之前的库存
                            stock.setStockQty(stock.getStockQty().add(byId.getRmRealQty()));
                            stock.updateById();

                        }

                        //新仓库加减
                        Stock stockNew = stockService.lambdaQuery().eq(Stock::getStockId, kingDeePurchaseStockOutItem.getStockId())
                                .eq(Stock::getMaterialId, kingDeePurchaseStockOutItem.getMaterialId()).last("limit 1").one();
                        //仓库有记录
                        if (stockNew != null){
                            //库存加减记录
                            stockAccLog("反审的退料单不同仓库减少",stockNew,kingDeePurchaseStockOutItem.getRmRealQty(),kingDeePurchaseStockOutItem.getFId(),false,"采购退料单");
                            //减少库存
                            stockNew.setStockQty(stockNew.getStockQty().subtract(kingDeePurchaseStockOutItem.getRmRealQty()));
                            stockNew.updateById();

                        }
                        //仓库库存记录无记录 不存在这种情况
                        if (stockNew == null){

                        }
                    }

                }


                // 入库表不能找到，查看库存表有无记录 ，无库存记录找库存，有库存记录相加
                if (byId == null){
                    //新仓库加减
                    Stock stockNew = stockService.lambdaQuery().eq(Stock::getStockId, kingDeePurchaseStockOutItem.getStockId())
                            .eq(Stock::getMaterialId, kingDeePurchaseStockOutItem.getMaterialId()).last("limit 1").one();
                    //仓库有记录
                    if (stockNew != null){
                        //库存加减记录
                        stockAccLog("退料单仓库减",stockNew,kingDeePurchaseStockOutItem.getRmRealQty(),kingDeePurchaseStockOutItem.getFId(),false,"采购退料单");
                        //加库存
                        stockNew.setStockQty(stockNew.getStockQty().subtract(kingDeePurchaseStockOutItem.getRmRealQty()));
                        stockNew.updateById();

                    }
                    //仓库库存记录无记录 不存在这种可能
                    if (stockNew == null){

                    }
                }
            }

            kingDeePurchaseStockOutItemService.saveOrUpdateBatch(kingDeePurchaseStockOuts);
        }

    }


    public List<KingDeePurchaseStockOutItem> allOutSync(String orgId, String orgNumber, String startTime, String endTime) throws Exception {
        // 调用入库单接口
        RequestApiBean request = k3Logic.getInstance(orgId, orgNumber, KingDeeCons.PURCHASE_RET_BILL, startTime, endTime);
        String filterStr = "FDocumentStatus = 'C' "+ k3Logic.supplySql("FApproveDate", request.getStartTime(), request.getEndTime());
        request.setFieldKeys("FID,FBillNo,FDate,FStockOrgId,FPurchaseOrgId,FSupplierId,FApproverId.FName,FApproveDate,FCancelDate," +
                "FDESCRIPTION,FDocumentStatus");
        request.setFilterString(String.format(filterStr, orgNum));
        request.setOrderString("FID ASC");
        request.setNotTime(true);

        String[] fields = new String[]{"id","billNo","fDate","stockOrgId","purchaseOrgId","supplierId","approverId","approveDate","cancelDate",
                "abcdRemarks","fdDocumentStatus"};
        List<KingDeePurchaseStockOut> objects = k3Logic.allSync(request, fields, KingDeePurchaseStockOut.class);
        if (!org.springframework.util.CollectionUtils.isEmpty(objects)) {
            kingDeePurchaseStockOutService.saveOrUpdateBatch(objects);
        }

        return allSyncOutOrder(orgId,orgNumber,startTime,endTime);
    }


    public List<KingDeePurchaseStockOutItem> allSyncOutOrder(String orgId, String orgNumber, String startTime, String endTime) throws Exception {
        // 调用入库单接口
        RequestApiBean request = k3Logic.getInstance(orgId, orgNumber, KingDeeCons.PURCHASE_RET_BILL, startTime, endTime);
        String filterStr = "FDocumentStatus = 'C' "+ k3Logic.supplySql("FApproveDate", request.getStartTime(), request.getEndTime());
        request.setFieldKeys("FPURMRBENTRY_FEntryID,FID,FBillNo,FMaterialId,FMaterialId.FNumber,FMaterialId.FName," +
                "FStockId,FNote,FRMMUSTQTY,FRMREALQTY,FREPLENISHQTY,FKEAPAMTQTY");
        request.setFilterString(String.format(filterStr, orgNumber));
        request.setOrderString("FID ASC");

        String[] fields = new String[]{"id","fId","billNo","materialId","materialNumber","materialName","stockId","note","rmMustQty","rmRealQty","replenishQty","keapAmtQty"};
        List<KingDeePurchaseStockOutItem> objects = k3Logic.allSync(request, fields, KingDeePurchaseStockOutItem.class);
/*        if (CollectionUtil.isNotEmpty(objects)) {
            kingDeePurchaseStockInItemService.saveOrUpdateBatch(objects);
        }*/
        return objects;
    }



    @Transactional(rollbackFor = Exception.class)
    public void purchaseStockOutQuanLiang() throws Exception {
        //全量更新
        // 调用退料单接口
        List<KingDeePurchaseStockOut> kingDeePurchaseStockOuts = kingDeePurchaseStockOutService.lambdaQuery().eq(KingDeePurchaseStockOut::getFdDocumentStatus, "C")
                .between(KingDeePurchaseStockOut::getApproveDate,DateUtil.offset(new Date(), DateField.MONTH, -1),new Date())
                .select(KingDeePurchaseStockOut::getId).list();
        if (CollectionUtil.isEmpty(kingDeePurchaseStockOuts))
            return;
        //获得所有通过的单
        String orderId="";
        for (KingDeePurchaseStockOut kingDeePurchaseStockOut : kingDeePurchaseStockOuts) {
            if (StringUtils.isEmpty(orderId)) {
                orderId = kingDeePurchaseStockOut.getId();
                continue;
            }
            orderId = orderId+","+kingDeePurchaseStockOut.getId();
        }
        RequestApiBean request = k3Logic.getInstance(orgId, orgNum, KingDeeCons.PURCHASE_RET_BILL, "2002-09-19 16:55:01", "2062-09-19 16:55:01");

        //查找被弃审的单
        String filterStr = "FDocumentStatus != 'C'"+ " AND FID IN ("+orderId+")";
        request.setFieldKeys("FID,FBillNo,FDate,FStockOrgId,FPurchaseOrgId,FSupplierId,FApproverId.FName,FApproveDate,FCancelDate," +
                "FDESCRIPTION,FDocumentStatus");
        request.setFilterString(String.format(filterStr, orgNum));
        request.setOrderString("FID ASC");

        String[] fields = new String[]{"id","billNo","fDate","stockOrgId","purchaseOrgId","supplierId","approverId","approveDate","cancelDate",
                "abcdRemarks","fdDocumentStatus"};
        request.setNotTime(true);
        List<KingDeePurchaseStockOut> objects = k3Logic.allSync(request, fields, KingDeePurchaseStockOut.class);
        /**
         *  全量查询表中不为已审核状态的单，退回数量，再删除数据，等下次审核
         * */
        if (CollectionUtil.isEmpty(objects))
            return;
        for (KingDeePurchaseStockOut object : objects) {
            //被弃审的返回数量
            List<KingDeePurchaseStockOutItem> list = kingDeePurchaseStockOutItemService.lambdaQuery().eq(KingDeePurchaseStockOutItem::getFId, object.getId()).list();
            for (KingDeePurchaseStockOutItem kingDeePurchaseStockOutItem : list) {
                //查出被反审的内容
                Stock stock = stockService.lambdaQuery().eq(Stock::getStockId, kingDeePurchaseStockOutItem.getStockId())
                        .eq(Stock::getMaterialId, kingDeePurchaseStockOutItem.getMaterialId()).last("limit 1").one();
                //库存减记录
                stockAccLog("全量反审的采购退料单仓库加",stock,kingDeePurchaseStockOutItem.getRmRealQty(),kingDeePurchaseStockOutItem.getFId(),true,"采购退料单");
                stock.setStockQty(stock.getStockQty().add(kingDeePurchaseStockOutItem.getRmRealQty()));
                stock.updateById();
                kingDeePurchaseStockOutItemService.removeById(kingDeePurchaseStockOutItem.getId());
            }
            kingDeePurchaseStockOutService.removeById(object.getId());
        }

    }


    @Transactional(rollbackFor = Exception.class)
    public void purchaseStockInQuanLiang() throws Exception {
        //全量更新
        // 调用入库单接口
        List<KingDeePurchaseStockIn> kingDeePurchaseStockIns = kingDeePurchaseStockInService.lambdaQuery().eq(KingDeePurchaseStockIn::getFdDocumentStatus, "C")
                .between(KingDeePurchaseStockIn::getApproveDate,DateUtil.offset(new Date(), DateField.MONTH, -1),new Date())
                .select(KingDeePurchaseStockIn::getId).list();
        if (CollectionUtil.isEmpty(kingDeePurchaseStockIns))
            return;
        //获得所有通过的单
        String orderId="";
        for (KingDeePurchaseStockIn kingDeePurchaseStockIn : kingDeePurchaseStockIns) {
            if (StringUtils.isEmpty(orderId)) {
                orderId = kingDeePurchaseStockIn.getId();
                continue;
            }
            orderId = orderId+","+kingDeePurchaseStockIn.getId();
        }
        RequestApiBean request = k3Logic.getInstance(orgId, orgNum, KingDeeCons.PURCHASE_IN_STOCK, "2002-09-19 16:55:01", "2062-09-19 16:55:01");

        //查找被弃审的单
        String filterStr = "FDocumentStatus != 'C'"+ " AND FID IN ("+orderId+")";
        request.setFieldKeys("FID,FBillNo,FDate,FStockOrgId,FDemandOrgId,FPurchaseOrgId,FSupplierId,FStockDeptId,FApproverId.FName,FApproveDate,FCancellerId,FCancelDate," +
                "FSupplyId.FName,FDocumentStatus");
        request.setFilterString(String.format(filterStr, orgNum));
        request.setOrderString("FID ASC");

        String[] fields = new String[]{"id","billNo","fDate","stockOrgId","demandOrgId","purchaseOrgId","supplierId","stockDeptId","approverId","approveDate","cancellerId","cancelDate",
                "supplyName","fdDocumentStatus"};
        request.setNotTime(true);
        List<KingDeePurchaseStockIn> objects = k3Logic.allSync(request, fields, KingDeePurchaseStockIn.class);
        /**
         *  全量查询表中不为已审核状态的单，退回数量，再删除数据，等下次审核
         * */
        if (CollectionUtil.isEmpty(objects))
            return;
        for (KingDeePurchaseStockIn object : objects) {
            //被弃审的返回数量
            List<KingDeePurchaseStockInItem> list = kingDeePurchaseStockInItemService.lambdaQuery().eq(KingDeePurchaseStockInItem::getFId, object.getId()).list();
            for (KingDeePurchaseStockInItem kingDeePurchaseStockInItem : list) {
                //查出被反审的内容
                Stock stock = stockService.lambdaQuery().eq(Stock::getStockId, kingDeePurchaseStockInItem.getStockId())
                        .eq(Stock::getMaterialId, kingDeePurchaseStockInItem.getMaterialId()).last("limit 1").one();
                //库存减记录
                stockAccLog("全量反审的入库单仓库减",stock,kingDeePurchaseStockInItem.getRealQty(),kingDeePurchaseStockInItem.getFId(),false,"采购入库单");
                stock.setStockQty(stock.getStockQty().subtract(kingDeePurchaseStockInItem.getRealQty()));
                stock.updateById();
                kingDeePurchaseStockInItemService.removeById(kingDeePurchaseStockInItem.getId());
            }
            kingDeePurchaseStockInService.removeById(object.getId());
        }

    }




    @Transactional(rollbackFor = Exception.class)
    public void purchaseStockIn() throws Exception {
        //1采购入库单入库库存
        List<KingDeePurchaseStockInItem> kingDeePurchaseStockInItems = allSync(orgId, orgNum, null, null);

        if (CollectionUtil.isNotEmpty(kingDeePurchaseStockInItems)) {
            for (KingDeePurchaseStockInItem kingDeePurchaseStockInItem : kingDeePurchaseStockInItems) {
                KingDeePurchaseStockInItem byId = kingDeePurchaseStockInItemService.getById(kingDeePurchaseStockInItem.getId());
                // 入库表表中能找到，相减加库存
                if (byId != null){

                    Stock stock = stockService.lambdaQuery().eq(Stock::getStockId, byId.getStockId())
                            .eq(Stock::getMaterialId, byId.getMaterialId()).last("limit 1").one();

                    //仓库相同  相减加减库存
                    if (byId.getStockId().equals(kingDeePurchaseStockInItem.getStockId())){

                        //加库存数量
                        BigDecimal subtract = kingDeePurchaseStockInItem.getRealQty().subtract(byId.getRealQty());

                        //仓库库存记录有记录
                        if (stock != null){
                            //库存加减记录
                            stockAccLog("反审的入库单相同仓库加减",stock,
                                    subtract.compareTo(BigDecimal.ZERO) < 0 ?(subtract.multiply(new BigDecimal("-1"))):subtract,kingDeePurchaseStockInItem.getFId(),subtract.compareTo(BigDecimal.ZERO) < 0 ?false:true,"采购入库单");
                            stock.setStockQty(stock.getStockQty().add(subtract));
                            stock.updateById();
                        }
                        //仓库库存记录无记录  不会发生
                        if (stock == null){

                        }
                    }

                    //仓库不同 先扣回之前仓库，再加现库存
                    if (!byId.getStockId().equals(kingDeePurchaseStockInItem.getStockId())){

                        //减除之前仓库
                        if (stock != null){
                            //库存加减记录
                            stockAccLog("反审的入库单不同仓库减",stock,byId.getRealQty(),kingDeePurchaseStockInItem.getFId(),false,"采购入库单");
                            //减少回之前的库存
                            stock.setStockQty(stock.getStockQty().subtract(byId.getRealQty()));
                            stock.updateById();

                        }

                        //新仓库加减
                        Stock stockNew = stockService.lambdaQuery().eq(Stock::getStockId, kingDeePurchaseStockInItem.getStockId())
                                .eq(Stock::getMaterialId, kingDeePurchaseStockInItem.getMaterialId()).last("limit 1").one();
                        //仓库有记录
                        if (stockNew != null){
                            //加库存
                            //库存加减记录
                            stockAccLog("反审的入库单不同仓库加",stockNew,kingDeePurchaseStockInItem.getRealQty(),kingDeePurchaseStockInItem.getFId(),true,"采购入库单");

                            stockNew.setStockQty(stockNew.getStockQty().add(kingDeePurchaseStockInItem.getRealQty()));
                            stockNew.updateById();
                        }
                        //仓库库存记录无记录
                        if (stockNew == null){
                            Stock stockIn = stockId(kingDeePurchaseStockInItem.getStockId(),kingDeePurchaseStockInItem.getMaterialId());

                            //库存加减记录
                            stockAccLog("反审的入库单不同仓库加",stockIn,kingDeePurchaseStockInItem.getRealQty(),kingDeePurchaseStockInItem.getFId(),true,"采购入库单");

                            //加库存
                            stockIn.setStockQty(kingDeePurchaseStockInItem.getRealQty());
                            stockIn.insert();
                        }
                    }

                }


                // 入库表不能找到，查看库存表有无记录 ，无库存记录找库存，有库存记录相加
                if (byId == null){
                    //新仓库加减
                    Stock stockNew = stockService.lambdaQuery().eq(Stock::getStockId, kingDeePurchaseStockInItem.getStockId())
                            .eq(Stock::getMaterialId, kingDeePurchaseStockInItem.getMaterialId()).last("limit 1").one();
                    //仓库有记录
                    if (stockNew != null){
                        //库存加减记录
                        stockAccLog("入库单仓库加",stockNew,kingDeePurchaseStockInItem.getRealQty(),kingDeePurchaseStockInItem.getFId(),true,"采购入库单");

                        //加库存
                        stockNew.setStockQty(stockNew.getStockQty().add(kingDeePurchaseStockInItem.getRealQty()));
                        stockNew.updateById();
                                       }
                    //仓库库存记录无记录
                    if (stockNew == null){
                        Stock stockIn = stockId( kingDeePurchaseStockInItem.getStockId(),kingDeePurchaseStockInItem.getMaterialId());
                        stockIn.setStockQty(new BigDecimal("0"));
                        //库存加减记录
                        stockAccLog("入库单不同仓库加",stockIn,kingDeePurchaseStockInItem.getRealQty(),kingDeePurchaseStockInItem.getFId(),true,"采购入库单");
                        //加库存
                        stockIn.setStockQty(kingDeePurchaseStockInItem.getRealQty());
                        stockIn.insert();

                    }
                }
            }

            kingDeePurchaseStockInItemService.saveOrUpdateBatch(kingDeePurchaseStockInItems);
        }

    }


    public List<KingDeePurchaseStockInItem> allSync(String orgId, String orgNumber, String startTime, String endTime) throws Exception {
        // 调用入库单接口
        RequestApiBean request = k3Logic.getInstance(orgId, orgNumber, KingDeeCons.PURCHASE_IN_STOCK, startTime, endTime);
        String filterStr = "FDocumentStatus = 'C' "+ k3Logic.supplySql("FApproveDate", request.getStartTime(), request.getEndTime());
        request.setFieldKeys("FID,FBillNo,FDate,FStockOrgId,FDemandOrgId,FPurchaseOrgId,FSupplierId,FStockDeptId,FApproverId.FName,FApproveDate,FCancellerId,FCancelDate," +
                "FSupplyId.FName,FDocumentStatus");
        request.setFilterString(String.format(filterStr, orgNumber));
        request.setOrderString("FID ASC");
        request.setNotTime(true);

        String[] fields = new String[]{"id","billNo","fDate","stockOrgId","demandOrgId","purchaseOrgId","supplierId","stockDeptId","approverId","approveDate","cancellerId","cancelDate",
                "supplyName","fdDocumentStatus"};
        List<KingDeePurchaseStockIn> objects = k3Logic.allSync(request, fields, KingDeePurchaseStockIn.class);
        if (!org.springframework.util.CollectionUtils.isEmpty(objects)) {
            kingDeePurchaseStockInService.saveOrUpdateBatch(objects);
        }

        return allSyncOrder(orgId,orgNumber,startTime,endTime);
    }


    public List<KingDeePurchaseStockInItem> allSyncOrder(String orgId, String orgNumber, String startTime, String endTime) throws Exception {
        // 调用入库单接口
        RequestApiBean request = k3Logic.getInstance(orgId, orgNumber, KingDeeCons.PURCHASE_IN_STOCK, startTime, endTime);
        String filterStr = "FDocumentStatus = 'C' "+ k3Logic.supplySql("FApproveDate", request.getStartTime(), request.getEndTime());
        request.setFieldKeys("FInStockEntry_FEntryID,FID,FBillNo,FMaterialId,FMaterialId.FNumber,FMaterialId.FName,FStockId,FNote,FMustQty,FRealQty,FUnitId.FName,FAuxUnitQty,FPrice,FEntryTaxRate" +
                ",FAmount,FTaxPrice,FEntryTaxAmount,FAllAmount,FUOM,FStockId.FName");
        request.setFilterString(String.format(filterStr, orgNumber));
        request.setOrderString("FID ASC");

        String[] fields = new String[]{"id","fId","billNo","materialId","materialNumber","materialName","stockId","note","mustQty","realQty","unit","auxUnitQty","price","entryTaxRate",
                "amount","taxPrice","entryTaxAmount","allAmount","uom","stockName"};
        List<KingDeePurchaseStockInItem> objects = k3Logic.allSync(request, fields, KingDeePurchaseStockInItem.class);
/*        if (CollectionUtil.isNotEmpty(objects)) {
            kingDeePurchaseStockInItemService.saveOrUpdateBatch(objects);
        }*/
        return objects;
    }

    @Transactional(rollbackFor = Exception.class)
    public void stockAccLog(String remark, Stock stock, BigDecimal number,String FID,Boolean type,String refType) {
        StockAcc stockAcc = new StockAcc();
        stockAcc.setStockId(stock.getStockId());
        stockAcc.setMaterialId(stock.getMaterialId());
        stockAcc.setChangeTime(new Date());
        int i = number.compareTo(BigDecimal.ZERO);
        stockAcc.setDirectFlag(type);
        stockAcc.setStockChangeQty(number);
        stockAcc.setStockCurQty(stock.getStockQty());
        stockAcc.setStockCloseQty(type == true?stock.getStockQty().add(number):stock.getStockQty().subtract(number));
        stockAcc.setRef(FID);
        stockAcc.setRefType(StringUtils.isNotEmpty(refType)?refType:"采购入库单");
        stockAcc.setUpdateTime(new Date());
        stockAcc.setNotes(remark);
        stockAcc.insert();
    }


    private Stock stockId(String  stockId,String materialId) throws Exception {
        // 调用即时库存
        RequestApiBean request = k3Logic.getInstance(null, null, KingDeeCons.STOCK_AMOUNT, null, null);

        String filterStr = "FStockId = "+ stockId+"AND FMaterialId="+materialId;
        request.setFieldKeys("FID,FStockId,FMaterialId,FBaseQty,FStockOrgId");
        request.setStartTime(null);
        request.setEndTime(null);
        request.setFilterString(filterStr);
        request.setNotTime(true);
        request.setOrderString("");

        String[] fields = new String[]{"id", "stockId", "materialId", "stockQty", "orgId"};
        List<Stock> objects = k3Logic.allSync(request, fields, Stock.class);
        if (CollectionUtil.isNotEmpty(objects)) {
            return objects.get(0);

        }else {
            throw new RemoteServiceException("数据为空，异常");
        }
    }

//    public void stockInSyncOrder() throws Exception {
//        // 调用即时库存
//        RequestApiBean request = k3Logic.getInstance(null, null, KingDeeCons.STOCK_AMOUNT, null, null);
//        String filterStr = "";
//        request.setFieldKeys("FID,FStockId,FMaterialId,FBaseQty,FStockOrgId");
//        request.setStartTime(null);
//        request.setEndTime(null);
//        request.setFilterString("");
//        request.setOrderString("");
//
//        String[] fields = new String[]{"id", "stockId", "materialId", "stockQty", "orgId"};
//        List<Stock> objects = k3Logic.allSync(request, fields, Stock.class);
//        if (CollectionUtil.isNotEmpty(objects)) {
//            stockService.saveOrUpdateBatch(objects);
//
//        }else {
//            throw new RemoteServiceException("数据为空，异常");
//        }
//    }
}
