package com.zfire.mall.manager.schedule;

import cn.hutool.core.collection.CollectionUtil;
import com.zfire.mall.manager.bean.trade.TradeLoginCountBean;
import com.zfire.mall.manager.commonmapper.LoginOrderMapper;
import com.zfire.mall.manager.plus.entity.CustomerBillLimitConfig;
import com.zfire.mall.manager.plus.entity.Policy;
import com.zfire.mall.manager.plus.service.CustomerBillLimitConfigService;
import com.zfire.mall.manager.plus.service.PolicyService;
import com.zfire.mall.manager.plus.service.ProductPriceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;

@Component
@Slf4j
public class UpdateTradeOrderLimitSchedule {

    @Resource
    CustomerBillLimitConfigService customerBillLimitConfigService;
    @Resource
    LoginOrderMapper loginOrderMapper;

    @Scheduled(fixedDelay = 30 * 1000)
    @Transactional(rollbackFor = Exception.class)
    public void task() {
        List<CustomerBillLimitConfig> configs = customerBillLimitConfigService.list();

        Set<String> customerList = configs.stream()
                .map(CustomerBillLimitConfig::getCustomerId)
                .collect(Collectors.toSet());
        if (CollectionUtil.isNotEmpty(customerList)) {
            List<TradeLoginCountBean> beans = loginOrderMapper.updateLoginOrderLimit(customerList);
            Map<String, CustomerBillLimitConfig> configMap = configs.stream()
                    .collect(Collectors.toMap(CustomerBillLimitConfig::getCustomerId, Function.identity()));
            List<CustomerBillLimitConfig> updateList = new ArrayList<>();
            for (TradeLoginCountBean bean : beans) {
                if (configMap.containsKey(bean.getCustomerId())) {
                    CustomerBillLimitConfig config = configMap.get(bean.getCustomerId());
                    config.setActuallyHomeLoginLimit(bean.getHomeCount())
                            .setActuallySpanLoginLimit(bean.getSpanCount())
                            .setActuallyWorkLoginLimit(bean.getWorkCount());
                    updateList.add(config);
                }
            }
            if (CollectionUtil.isNotEmpty(updateList)) {
                customerBillLimitConfigService.saveOrUpdateBatch(updateList);
            }
        }
    }
}
